/******************************************************************************
 * 
 *  Title  : commandFunctionInit.c 
 *  Version 0.0
 *   
 *  Description: Initialization of array of pointers to command functions.
 *
 *  Author: Lukas Tomasek, tomasekl@fzu.cz
 *
 ******************************************************************************/

/******************************************************************************
 *                              Header files                                  *
 ******************************************************************************/

#include "commandFunctionInit.h"   

#include "commandFunc_readWriteVME.h"
#include "commandFunc_readWriteMaster.h" 
#include "commandFunc_readWriteSlave.h"		   
#include "commandFunc_sendPrimList.h"
#include "commandFunc_compareFiles.h"
#include "commandFunc_busTest.h"  
#include "commandFunc_runChildCommandList.h"   
#include "commandFunc_readWriteFpgaFlash.h"  
#include "commandFunc_writeMDSPFlash.h"   
#include "commandFunc_BOCSetup.h"    
#include "commandFunc_runExternalProgram.h" 
#include "commandFunc_delay.h" 

/******************************************************************************
 *                              Global functions                              *
 ******************************************************************************/

/*=============================================================================
 *                             commandFunctionInit()
 *=============================================================================
 *
 *  Initializes array of pointers to command functions.
 *
 */
 
void commandFunctionInit(COMMAND_FUNCTION_TYPE (*commandFunction[])){

	commandFunction[RW_VME_COMMAND_ID]=&commandFunction_readWriteVME;
	commandFunction[RW_MASTER_COMMAND_ID]=&commandFunction_readWriteMaster; 
	commandFunction[RW_SLAVE_COMMAND_ID]=&commandFunction_readWriteSlave; 
	commandFunction[SEND_PRIM_LIST_COMMAND_ID]=&commandFunction_sendPrimList; 
	commandFunction[COMPARE_FILES_COMMAND_ID]=&commandFunction_compareFiles; 
	commandFunction[BUS_TEST_ID]=&commandFunction_busTest;
	commandFunction[RUN_CHILD_COMMAND_LIST_ID]=&commandFunction_runChildCommandList;  
	commandFunction[RW_FPGA_FLASH_ID]=&commandFunction_readWriteFpgaFlash;
	commandFunction[WRITE_MDSP_FLASH_ID]=&commandFunction_writeMDSPFlash;
	commandFunction[BOC_SETUP_ID]=&commandFunction_BOCSetup; 
	commandFunction[RUN_EXTERNAL_PROGRAM_ID]=&commandFunction_runExternalProgram;  
	commandFunction[DELAY_COMMAND_ID]=&commandFunction_delay;  
	
	return;
}

/******************************************************************************/
