/******************************************************************************
 * 
 *  Title  : commandParamsStruct.h 
 *  Version 0.0
 *   
 *  Description:  Command ids and common params union definition.
 *  Related files: commandListDefinitions.h etc.
 *
 *  Author: Lukas Tomasek, tomasekl@fzu.cz
 *
 ******************************************************************************/

#ifndef COMMAND_PARAMS_STRUCT_H	 /* multiple inclusion protection */
#define COMMAND_PARAMS_STRUCT_H
 
/******************************************************************************
 *                              Header files                                  *
 ******************************************************************************/
 
#include "commandFunc_readWriteVME.h"
#include "commandFunc_readWriteMaster.h" 
#include "commandFunc_readWriteSlave.h"		   
#include "commandFunc_sendPrimList.h"
#include "commandFunc_compareFiles.h" 
#include "commandFunc_busTest.h" 
#include "commandFunc_runChildCommandList.h"     
#include "commandFunc_readWriteFpgaFlash.h" 
#include "commandFunc_writeMDSPFlash.h"   
#include "commandFunc_runExternalProgram.h" 
#include "commandFunc_delay.h"    

/******************************************************************************
 *                               Definitions                                  *
 ******************************************************************************/

/* command IDs */
/* Note: Each command must have unique number!! Any new command ID should
         be  old_LAST_COMMAND_ID +1; no "gaps", please */
#define RW_VME_COMMAND_ID 			0  
#define RW_MASTER_COMMAND_ID 		1  
#define RW_SLAVE_COMMAND_ID 		2 
#define SEND_PRIM_LIST_COMMAND_ID 	3
#define COMPARE_FILES_COMMAND_ID 	4
#define BUS_TEST_ID 				5   
#define RUN_CHILD_COMMAND_LIST_ID 	6 
#define RW_FPGA_FLASH_ID			7 
#define WRITE_MDSP_FLASH_ID			8
#define BOC_SETUP_ID     			9
#define RUN_EXTERNAL_PROGRAM_ID     10 
#define DELAY_COMMAND_ID       		11

#define LAST_COMMAND_ID	 (DELAY_COMMAND_ID)

/******************************************************************************
 *                           Type definitions                                 *
 ******************************************************************************/

union COMMAND_PARAMS_UNION{  /* used in commandListDefinitions.h */

	struct RW_VME_PARAMS readWriteVme;
	struct RW_MASTER_PARAMS readWriteMaster; 
	struct RW_SLAVE_PARAMS readWriteSlave; 
	struct SEND_PRIM_LIST_PARAMS sendPrimList; 
	struct COMPARE_FILES_PARAMS compareFiles;   
	struct BUS_TEST_PARAMS busTest;   
	struct RUN_CHILD_COMMAND_LIST_PARAMS runChildCmdList;
	struct RW_FPGA_FLASH_PARAMS readWriteFpgaFlash;   
	struct W_MDSP_FLASH_PARAMS writeMDSPFlash;
	//struct BOC_SETUP_PARAMS BOCSetup;
	struct RUN_EXTERNAL_PROGRAM_PARAMS runExternalProgram;
	struct DELAY_PARAMS delay; 
	

};

/******************************************************************************/ 

#endif /* COMMAND_PARAMS_STRUCT_H */

