/******************************************************************************
 * 
 *  Title  : primFunc_setLed.c 
 *  Version 0.0,
 *   
 *  Description: Set Led primitive function.    
 *  Related files:
 *
 *  Author: Lukas Tomasek, tomasekl@fzu.cz
 *
 ******************************************************************************/

/******************************************************************************
 *                              Header files                                  *
 ******************************************************************************/

#include <stdio.h>
#include "primFunc_setLed.h"
#include "fileUtility.h"
#include "globalDefinitions.h"
#include "uirUtility.h"
#include "primParamsUir.h"
#include "mainUir.h"
#include <utility.h>   
#include "primFunctionInit.h"

/******************************************************************************
 *                              Global functions                              *
 ******************************************************************************/

/*=============================================================================
 *                              primFunction_setLed()
 *=============================================================================
 *
 *	 
 *
 */
ERROR_ID primFunction_setLed(PRIM_FUNC_OPTION funcOption, struct PRIM_TABLE *primTable, void *primitive, UINT32 inputParameter, UINT8 slotNumber){

	FILE *file;
	
	struct MSG_HEAD* primHeader=(struct MSG_HEAD*) primitive;
	UINT32 *primDataPtr=(UINT32*)((UINT32)primHeader+sizeof(struct MSG_HEAD));
	
	ERROR_ID errorId=SUCCESS;
	char errorMessage[200];
	int status;
	
	UINT32 primBodyLength;
	UINT32 primitiveId;

	struct SET_LED_IN *setLed=&primTable->params.setLed;
	const int panel=global.panel.primEdit[getPrimArrayId(SET_LED)];
	UINT32 i;
	UINT32 dataLength;
	struct SET_LED_IN *setLedIn;
	struct SET_LED_OUT *setLedOut;
	char textMessage[200];
	char errorText[100]; 
	UINT32 listIndex;
	int ringIndex;

		
	if(R_SET_LED!=R_SET_LED_HOST){
		ERROR_CHECK(PROGRAM_ERROR, primitive SET_LED - wrong revision number);
		return(PROGRAM_ERROR);
	}

	switch(funcOption){
		case PRIM_PANEL_TO_TABLE:
		
			GetCtrlVal(panel, SET_LEDP_LED_STATE, &setLed->ledState);  
			GetCtrlVal(panel, SET_LEDP_NUMBER, &setLed->ledNum);
			
			/* NO BREAK HERE!!!*/
		case PRIM_PARAMS_TO_TABLE:				
			/* add table headers - don't set indexes!!! */  
			primTable->inPrimHeader.length=SIZEOF(struct SET_LED_IN)+SIZEOF(struct MSG_HEAD); 
			primTable->inPrimHeader.id=SET_LED;
			primTable->inPrimHeader.primRevision=R_SET_LED; 
			primTable->replyLength=0;
		
	  		break;
		
		case PRIM_TABLE_TO_PANEL:
			if(primTable->inPrimHeader.primRevision!=R_SET_LED){  
				ERROR_CHECK(PROGRAM_ERROR, primitive SET_LED - wrong revision number);
				return(PROGRAM_ERROR);
			}
			
			SetCtrlVal(panel, SET_LEDP_LED_STATE, setLed->ledState);    
			SetCtrlVal(panel, SET_LEDP_NUMBER, setLed->ledNum);
			
			break;
		case PRIM_BUILD:
 			/* copy mask */
			*(struct SET_LED_IN*)(primDataPtr)=*setLed;
		
 			break;
 
		case REP_DATA_PROCESS:
		
			/* no reply */
			
			break;
		
		case WRITE_TO_FILE:
			
			file=(FILE*)inputParameter;
			
  			fprintf(file, "ledNum= %d, ledState= %d\n", setLed->ledNum, setLed->ledState); 
  			
			break;
			
		case READ_FROM_FILE:
		
			file=(FILE*)inputParameter;
			
  			status=fscanf(file, "ledNum= %d, ledState= %d\n", &setLed->ledNum, &setLed->ledState);
  			if(status==-1) {ERROR_CHECK(FATAL_ERROR, Read from prim file error!); return(FATAL_ERROR);}		
         	
			break;	
	
		
		default: ;//error	
 	}

	return(SUCCESS);	
			
}

/******************************************************************************/ 

