/******************************************************************************
 * 
 *  Title  : primFunc_writeBuffer.c 
 *  Version 0.0,
 *   
 *  Description: Histogram control primitive function.    
 *  Related files:
 *
 *  Author: Lukas Tomasek, tomasekl@fzu.cz
 *
 ******************************************************************************/

/******************************************************************************
 *                              Header files                                  *
 ******************************************************************************/

#include <stdio.h>
#include "primFunc_writeBuffer.h"
#include "globalDefinitions.h"
#include "uirUtility.h"
#include "primParamsUir.h"
#include "mainUir.h"
#include <utility.h>   
#include "primFunctionInit.h"
#include "RWlists.h"

/******************************************************************************
 *                              Global functions                              *
 ******************************************************************************/

/*=============================================================================
 *                              primFunction_writeBuffer()
 *=============================================================================
 *
 *	  
 *
 */
ERROR_ID primFunction_writeBuffer(PRIM_FUNC_OPTION funcOption, struct PRIM_TABLE *primTable, void *primitive, UINT32 inputParameter, UINT8 slotNumber){

	FILE *file;
	
	struct MSG_HEAD* primHeader=(struct MSG_HEAD*) primitive;
	UINT32 *primDataPtr=(UINT32*)((UINT32)primHeader+sizeof(struct MSG_HEAD));
	
	ERROR_ID errorId=SUCCESS;
	char errorMessage[200];
	int status;
	
	UINT32 primBodyLength;
	UINT32 primitiveId;

	struct WRITE_BUFFER_IN *writeBuffer=&primTable->params.writeBuffer;
	const int panel=global.panel.primEdit[getPrimArrayId(WRITE_BUFFER)];
	UINT32 i;
	UINT32 dataLength;
	char textMessage[200];
	char errorText[100]; 
	UINT32 listIndex;
	int ringIndex;
	char str[300];
	
		
	if(R_WRITE_BUFFER!=R_WRITE_BUFFER_HOST){
		ERROR_CHECK(PROGRAM_ERROR, primitive WRITE_BUFFER - wrong revision number);
		return(PROGRAM_ERROR);
	}

	switch(funcOption){
		case PRIM_PANEL_TO_TABLE:
		
			GetCtrlVal(panel, WR_BUFFP_BUFFER, &writeBuffer->buffer);
			GetCtrlVal(panel, WR_BUFFP_STRING, str);
			strncpy (writeBuffer->string, str, 63);
			writeBuffer->string[63]=0;

			/* NO BREAK HERE!!!*/
		case PRIM_PARAMS_TO_TABLE:				
			/* add table headers - don't set indexes!!! */  
			primTable->inPrimHeader.length=SIZEOF(struct WRITE_BUFFER_IN)+SIZEOF(struct MSG_HEAD); 
			primTable->inPrimHeader.id=WRITE_BUFFER;
			primTable->inPrimHeader.primRevision=R_WRITE_BUFFER; 
			primTable->replyLength=0;
		
	  		break;
		
		case PRIM_TABLE_TO_PANEL:
			if(primTable->inPrimHeader.primRevision!=R_WRITE_BUFFER){  
				ERROR_CHECK(PROGRAM_ERROR, primitive WRITE_BUFFER - wrong revision number);
				return(PROGRAM_ERROR);
			}
			
			SetCtrlVal(panel, WR_BUFFP_BUFFER, writeBuffer->buffer);
			SetCtrlVal(panel, WR_BUFFP_STRING, writeBuffer->string);
			

			break;
		case PRIM_BUILD:
 			/* copy data */
			*(struct WRITE_BUFFER_IN*)(primDataPtr)=*writeBuffer;
		
 			break;
 
		case REP_DATA_PROCESS:
		
			/* no reply */
			
			break;

		case WRITE_TO_FILE:

			file=(FILE*)inputParameter;
			
  			fprintf(file, "slaveNumber= %d\n", writeBuffer->buffer);
  			fprintf(file, "string[]= %s\n", writeBuffer->string);
		
			break;
			
		case READ_FROM_FILE:
		
			file=(FILE*)inputParameter;
			
			status=fscanf(file, "slaveNumber= %d\n", &writeBuffer->buffer);
			if(status==-1) {ERROR_CHECK(FATAL_ERROR, Read from prim file error!); return(FATAL_ERROR);}
			status=readLineFromIndex(file, strlen("string[]= "), PATHNAME_LENGTH, writeBuffer->string);
			if(status!=0) {ERROR_CHECK(FATAL_ERROR, Read from prim file error!); return(FATAL_ERROR);}
  			
			break;	
	
		
		default: ;//error	
 	}

	return(SUCCESS);	
			
}

/******************************************************************************/ 

