/******************************************************************************
 *
 *  Title  : primParamsStruct.h
 *  Version 0.0
 *
 *  Description: primFunc_PARAMS_UNION definition
 *  Related files: listDefinitions.h
 *
 *  Author: Lukas Tomasek, tomasekl@fzu.cz
 *
 ******************************************************************************/

#ifndef PRIM_PARAMS_STRUCT_H	 /* multiple inclusion protection */
#define PRIM_PARAMS_STRUCT_H

/******************************************************************************
 *                              Header files                                  *
 ******************************************************************************/

/* common */
#include "primFunc_echo.h"
#include "primFunc_setErrmsgMask.h"
#include "primFunc_pauseList.h"
#include "primFunc_eventTrapSetup.h"
#include "primFunc_setMemory.h"
#include "primFunc_copyMemory.h"
#include "primFunc_memoryTest.h"
#include "primFunc_setLed.h"
#include "primFunc_flashLed.h"
#include "primFunc_sendData.h"
#include "primFunc_moduleMask.h"
#include "primFunc_setTrigger.h"
#include "primFunc_startTask.h"
#include "primFunc_taskOperation.h"
#include "primFunc_test.h"
#include "primFunc_writeBuffer.h"
#if   defined(SCT_ROD)
	 /* common SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* common pixel specific prims */

#endif /* SCT_ROD or PIXEL_ROD */


/* slave */
#include "primFunc_startEventTrapping.h"
#include "primFunc_stopEventTrapping.h"
#include "primFunc_histogramSetup.h"

#if   defined(SCT_ROD)
	 /* slave SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* slave pixel specific prims */

#endif /* SCT_ROD or PIXEL_ROD */

/* master */
#include "primFunc_RWSlaveMemory.h"
#include "primFunc_transSerData.h"
#include "primFunc_startSlaveExecuting.h"
#include "primFunc_configSlave.h"
#include "primFunc_RWRegField.h"
#include "primFunc_pollRegField.h"
#include "primFunc_RWFifo.h"
#include "primFunc_sendSlaveList.h"
#include "primFunc_startSlaveList.h"
#include "primFunc_slaveListOp.h"
#include "primFunc_buildStream.h"
#include "primFunc_sendStream.h"
#include "primFunc_RWmoduleData.h"
#include "primFunc_sendConfig.h"
#include "primFunc_dspReset.h"
#include "primFunc_setRodMode.h"
#include "primFunc_RWmoduleVariable.h"
#include "primFunc_RWBocData.h"  

#if   defined(SCT_ROD)
	 /* master SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* master pixel specific prims */
#include "primFunc_testGlobalReg.h"

#endif /* SCT_ROD or PIXEL_ROD */

/******************************************************************************
 *                           Type definitions                                 *
 ******************************************************************************/



union PRIM_PARAMS_UNION{
	/* common */
	struct PRIM_ECHO_PARAMS echo;
	struct SET_ERRMSG_MASK_IN errmsgMask;
	/* prim PAUSE_LIST has no params */
	struct EVENT_TRAP_SETUP_IN eventTrapSetup;
	struct SET_MEMORY_IN setMemory;
	struct COPY_MEMORY_IN copyMemory;
	struct MEMORY_TEST_IN memoryTest;
	struct SET_LED_IN setLed;
	struct FLASH_LED_IN flashLed;
	struct SEND_DATA_PARAMS sendData;
	struct MODULE_MASK_IN moduleMask;
	struct SET_TRIGGER_IN setTrigger;
	struct START_TASK_PARAMS startTask;
	struct TASK_OPERATION_IN taskOperation;
	struct TEST_PARAMS test;
	struct WRITE_BUFFER_IN writeBuffer;
#if   defined(SCT_ROD)
	 /* common SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* common pixel specific prims */
	
#endif /* SCT_ROD or PIXEL_ROD */
	
	/* slave */
	/* prim START_EVENT_TRAPPING has no input params */
	/* prim STOP_EVENT_TRAPPING has no input params */
	struct PRIM_HISTOGRAM_SETUP_PARAMS histogramSetup;
#if   defined(SCT_ROD)
	 /* slave SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* slave pixel specific prims */
	
#endif /* SCT_ROD or PIXEL_ROD */
	
	/* master */
	struct PRIM_RW_SLAVE_MEMORY_PARAMS RWSlaveMemory;
	struct PRIM_TRANS_SDATA_PARAMS transSerData;
	struct START_SLAVE_EXECUTING_IN startSlaveExec;
	struct CONFIG_SLAVE_IN configSlave;
	struct PRIM_RWFIELD_PARAMS RWField;
	struct PRIM_POLL_REG_FIELD_PARAMS pollRegField;
	struct PRIM_RWFIFO_PARAMS RWFifo;
	struct PRIM_SEND_SLAVE_LIST_PARAMS sendSlaveList;
	struct START_SLAVE_LIST_IN startSlaveList;
	struct SLAVE_LIST_OP_IN slaveListOp;
	struct BUILD_STREAM_PARAMS buildStream;
	struct SEND_STREAM_IN sendStream;
	struct RW_MODULE_DATA_PARAMS RWmodule;
	SendConfigIn sendConfig;
	struct DSP_RESET_IN dspReset;
	struct SET_ROD_MODE_IN setRodMode;
	//struct SET_ROD_MODE_PARAMS setRodMode;
	struct RW_MODULE_VARIABLE_PARAMS RWmoduleVar;
	struct RW_BOC_DATA_PARAMS RWBocData;
	struct BOC_HISTOGRAM_IN BOChisto;
	
#if   defined(SCT_ROD)
	 /* master SCT specific prims */   
	 
#elif defined(PIXEL_ROD)
	/* master pixel specific prims */
	
	struct TEST_GLOBAL_REG_PARAMS testGlobalReg;

#endif /* SCT_ROD or PIXEL_ROD */
	
	 

};

/******************************************************************************/

#endif /* PRIM_PARAMS_STRUCT_H */

