/* _CLK_REGS and _CLK_USETIMER must be defined here (LT 7/13/01) */
_CLK_REGS = 0;
_CLK_USETIMER = 0;

/* MEMORY sections are defined in the MEMORY directive below. These MUST be
   the same length as their counterparts in memoryPartitions.h or memory
   over-writes can potentially occur!! */

MEMORY  {
	IPRAM   : origin = 0x0,        len = 0x10000

	IDREGS  : origin = 0x80000000, len = 0x80
	IDATA   : origin = 0x80000080, len = 0x800
	IDRAM   : origin = 0x80000880, len = 0x7780
	BURST   : origin = 0x80008000, len = 0x8000

	XPROG   : origin = 0x02000000, len = 0x40000

	XFREE0_NC : origin = 0x02040000, len = 0x010c0000  /* non-cachable */
}

SECTIONS {
	GROUP : 0x0
	{
		.vectors   /* Vector table must be 1st in IPRAM */
		.text
	}

	.usect00  "icode"         > IPRAM
	.usect01  "xcode"         > XPROG
	.usect02  "xpdata"        > XPROG
	.usect03  "xtext"         > XTEXT

	.usect04  "xdata"         > XPROG
	.usect05  "fitluts"       > XPROG

	.usect06  "csl_ip"        > IPRAM
	.usect07  "csl_xp"        > XPROG
	.usect08  "rts_ip"        > IPRAM
	.usect09  "rts_math"      > IPRAM
	.usect10  "rts_stdio"     > XPROG

	.usect11  "commRegs"      > IDREGS
	.usect12  "idata"         > IDATA
	.usect13  "burst"         > BURST


	.usectH0  "iheap"      > IDRAM
	.usectH2  "xheap0nc"   > XFREE0_NC

	/* NOTE: cinit > XPROG means that code must be loaded by pre-loading the CPU EMIF
	   registers to enable external memory (the MDSP now does this), and then loading
	   the code with VME or MDSP. If it must be put in IDRAM, perhaps it can be put in
	   the burst buffer (cinit section initializes c variables at program start, and
	   so won't be needed later).
	   
	   Note that this is only relevant if the .cinit section is not pre-loaded by the
	   coff conversion program. If the linker is invoked with load-time initialization
	   (-cr flags), as is currently done, the .cinit section is made into a copy section
	   and not directly loaded into memory. Instead, it is processed and the variable
	   values within it are copied directly into their locations in memory (When the
	   run-time initialization is used, this is done by the TI auto_init routine). Load
	   time initialization is better since the look-up tables for efficient fitting
	   require a lot of space. */
	.cinit   > XPROG

	.const   > IDRAM
	.data    > IDRAM
	.switch  > IDRAM
	.pinit   > IDRAM
	.cio     > IDRAM
	.far     > IDRAM
	.sysmem  > IDRAM
	.bss     > IDRAM
	.stack   > IDRAM
}
