/************************************************************************************/
/*! \file  commTest.h
    \brief Contains the communications test structure.

   The communications test routine is for testing out the speed & accuracy of
   MDSP->SDSP communications. See sdspCommTest in commTest.c for more details.
*/
#ifndef COMM_TEST_H
#define COMM_TEST_H

//!  The Inter-DSP Communications Test Structure.
/*!
	The communications test is used to measure the performance of the MDSP to
	SDSP communications under various conditions (code position, etc.). It
	is used to optimize MDSP routines where code size & speed must be balanced.
	Generally the access is to & from the communications registers; any valid
	SDSP address can be used though. See commTest.c for a summary of some results.
*/
#define N_COMM_TEST_REGS 32
typedef struct SdspRegTest {
	uint32 reg[N_COMM_TEST_REGS];
} SdspRegTest;

typedef struct CommTest {
	uint32 codePos,  //!< Indicates where the code runs out of in MDSP memory.
	       codeType, //!< Indicates the code type: fxn calls, inlined fxn, monolithic.
	       slvBits,  //!< Bitfield describing which SDSPs to test on.
	       nTimes,   //!< The number of times to perform the test.
	       intLoop,  //!< How the looping should be done: outside or inside called fxn.
	       nWords,   //!< The number of words to read from the SDSP (<= 32)
	       sdspAddr, //!< The address to read from.
	       sdspSect, //!< The memory section of the structure storing the SDSP data.

	       check,    //!< Indcates that data integrity checks should be done. nyi
	       fBit;     //!< If set a read/modify bit/write/re-read & check will be done. nyi
} CommTest;

#endif /* Multiple inclusion protection */
