/************************************************************************************
 * peripheralMap.h: Defines mapping of available peripherals to their functions on
 *                  the master and slave DSPs.
 *
 *  Damon Fasching, UW Madison (510)486-5230               fasching@wisconsin.cern.ch
 ************************************************************************************/
#ifndef PERIPHERAL_MAP
#define PERIPHERAL_MAP

#include <csl.h>
#include <csl_irq.h>
#include <csl_dma.h>
#include <csl_timer.h>
#include <csl_mcbsp.h>

/* The two DSP timer channels are used to flash a hearbeat LED, and for general
   purpose timing information for all DSP functions. */
#define HEARTBEAT_TIMER_CH (TIMER_DEV0)
#define USER_TIMER_CH      (TIMER_DEV1)

#ifdef I_AM_SLAVE_DSP1
  
  /* This DMA channel is used to burst data from the router output event FIFO to the
   * slave burst buffer. */

  #define CH_DMA_DATA_BRST              (DMA_CH0)
  #define DEST_ADDR_DMA_DATA_BRST       (DMA0_DEST_ADDR)
  #define SECONDARY_CTRL_ADDR_DATA_BRST (DMA0_SECONDARY_CTRL_ADDR)

  /* This interrupt is generated when the DMA from the router is finished.  It is
   * hooked to an ISR which starts analysis of the data.  */
  
  #define IRQ_BLOCK_DMA_DONE     (IRQ_INT_08)      /* IRQ for end of frame done */

  /* MCBSP 0 drives the enable/disable router event trapping signal.
   * MCBSP 1 drives the HALT_OUTPUT and an LED */

  #define MCBSP_TRAP_ENABLE    0
  #define MCBSP_HALT_AND_LED   1

#endif

#endif
