
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------
//
//  "uiCmdListEditor.c"
//
//  Implementation file for UI build functions
//
//  Date: 11-07-2004
//
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//  Include header files
//----------------------------------------------------------------------------------------------------

#include "uiCmdListEditor.h"
#include <Utility.h> // definition of NULL
#include "toolbox.h" // definition of errChk macro

extern int CVIFUNC_C GetUserBitmapFromText (char *imageText, int version, int *bitmapId); // needed if there are bitmaps
extern int CVIFUNC_C NewActiveXCtrlFromPersistence (int panel, const char *ctrlLabel,
                                                    int top, int left, char *defaultIID,
                                                    int binaryFormat, char *persistenceText,
                                                    HRESULT *activeXError); // needed if there are ActiveX controls

//----------------------------------------------------------------------------------------------------
//  Prototype UI build functions.
//----------------------------------------------------------------------------------------------------

int buildPanel_cmdListEditor (int hParentPanel);
int buildMenu_cmdListEditor (int hParentPanel);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: CLIST_EDIT
// ------------------------------------------------------------------------------------------------

// Control: CLIST_EDIT_COMMAND_RING
int CVICALLBACK commandRing (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_CLEAR_LIST
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_CLEAR_CLCOMM
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_CLEAR_CCOMM
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_SAVE_LIST
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_LOAD_LIST
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_INSERT_LIST
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_SEND_CLIST_TO_ROD
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_REPLACE
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_ADD_AFTER
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_ADD_BEFORE
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_EDIT_COMMAND
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_DELETE_COMMAND
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: CLIST_EDIT_LIST_TABLE
int CVICALLBACK clistEditButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Menubar: CLIST_MENU
// ------------------------------------------------------------------------------------------------

// Menu: GOTO_MAIN
void CVICALLBACK commandListEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: PRIM_LIST_EDIT
void CVICALLBACK commandListEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: DATA_FILE_CREATOR
void CVICALLBACK commandListEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: CLOSE
void CVICALLBACK commandListEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);


//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  CLIST_EDIT
// ------------------------------------------------------------------------------------------------

int CLIST_EDIT_COMMAND_RING                  = 0;  // control identifier
int CLIST_EDIT_CLEAR_LIST                    = 0;  // control identifier
int CLIST_EDIT_CLEAR_CLCOMM                  = 0;  // control identifier
int CLIST_EDIT_CLEAR_CCOMM                   = 0;  // control identifier
int CLIST_EDIT_SAVE_LIST                     = 0;  // control identifier
int CLIST_EDIT_LOAD_LIST                     = 0;  // control identifier
int CLIST_EDIT_INSERT_LIST                   = 0;  // control identifier
int CLIST_EDIT_SEND_CLIST_TO_ROD             = 0;  // control identifier
int CLIST_EDIT_REPLACE                       = 0;  // control identifier
int CLIST_EDIT_ADD_AFTER                     = 0;  // control identifier
int CLIST_EDIT_ADD_BEFORE                    = 0;  // control identifier
int CLIST_EDIT_EDIT_COMMAND                  = 0;  // control identifier
int CLIST_EDIT_DELETE_COMMAND                = 0;  // control identifier
int CLIST_EDIT_LIST_TABLE                    = 0;  // control identifier
int CLIST_EDIT_SLOT_NUMBER                   = 0;  // control identifier
int CLIST_EDIT_COMMAND_REPEAT                = 0;  // control identifier
int CLIST_EDIT_COMMAND_COUNT                 = 0;  // control identifier
int CLIST_EDIT_CONT_IF_ERROR                 = 0;  // control identifier
int CLIST_EDIT_LAST_FILE                     = 0;  // control identifier
int CLIST_EDIT_CMD_COMMENT                   = 0;  // control identifier
int CLIST_EDIT_CLIST_COMMENT                 = 0;  // control identifier
int CLIST_EDIT_CLIST_REP                     = 0;  // control identifier
int CLIST_EDIT_DECORATION_3                  = 0;  // control identifier
int CLIST_EDIT_TEXTMSG_3                     = 0;  // control identifier
int CLIST_EDIT_TEXTMSG_4                     = 0;  // control identifier
int CLIST_EDIT_TEXTMSG                       = 0;  // control identifier
int CLIST_EDIT_TEXTMSG_5                     = 0;  // control identifier
int CLIST_EDIT_TEXTMSG_6                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Menubar:  CLIST_MENU
// ------------------------------------------------------------------------------------------------

int CLIST_MENU_GOTO_MAIN                     = 0;  // menu identifier
int CLIST_MENU_PRIM_LIST_EDIT                = 0;  // menu identifier
int CLIST_MENU_DATA_FILE_CREATOR             = 0;  // menu identifier
int CLIST_MENU_CLOSE                         = 0;  // menu identifier

//----------------------------------------------------------------------------------------------------
// buildPanel_cmdListEditor ():  Build UI object CLIST_EDIT; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_cmdListEditor (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "COMMAND LIST EDITOR", 29, 13, 676, 745);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "CLIST_EDIT");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XC9D3D0);
//  SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_OFFWHITE);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_GRAY);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_RAISED_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 10);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
//  SetPanelAttribute (hPanel, ATTR_MOVABLE, 1);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
//  SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
    SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 1);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: CLIST_EDIT_COMMAND_RING
    CLIST_EDIT_COMMAND_RING = NewCtrl (hPanel, CTRL_RING, "", 60, 8);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_CONSTANT_NAME, "COMMAND_RING");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_CALLBACK_FUNCTION_POINTER, commandRing);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_CTRL_MODE, VAL_NORMAL);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_COLOR, 0X202020);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_HEIGHT, 15);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_RAISED, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_BGCOLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_POINT_SIZE, 13);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_WIDTH, 353);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_TOP, 42);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_LEFT, 266);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: CLIST_EDIT_CLEAR_LIST
    CLIST_EDIT_CLEAR_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE LIST", 471, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_CONSTANT_NAME, "CLEAR_LIST");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_CLEAR_LIST);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_CLEAR_CLCOMM
    CLIST_EDIT_CLEAR_CLCOMM = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "Clear", 630, 425);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_CONSTANT_NAME, "CLEAR_CLCOMM");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_CLEAR_CLCOMM);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_WIDTH, 38);

    // Build control: CLIST_EDIT_CLEAR_CCOMM
    CLIST_EDIT_CLEAR_CCOMM = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "Clear", 630, 367);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_CONSTANT_NAME, "CLEAR_CCOMM");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_CLEAR_CCOMM);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_WIDTH, 38);

    // Build control: CLIST_EDIT_SAVE_LIST
    CLIST_EDIT_SAVE_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "SAVE LIST", 435, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_CONSTANT_NAME, "SAVE_LIST");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_SAVE_LIST);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_LOAD_LIST
    CLIST_EDIT_LOAD_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "LOAD LIST", 381, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_CONSTANT_NAME, "LOAD_LIST");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_LOAD_LIST);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_INSERT_LIST
    CLIST_EDIT_INSERT_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "INSERT LIST", 408, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_CONSTANT_NAME, "INSERT_LIST");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_INSERT_LIST);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_SEND_CLIST_TO_ROD
    CLIST_EDIT_SEND_CLIST_TO_ROD = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "SEND TO ROD", 113, 370);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_CONSTANT_NAME, "SEND_CLIST_TO_ROD");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_WIDTH, 92);

    // Build control: CLIST_EDIT_REPLACE
    CLIST_EDIT_REPLACE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "REPLACE>>", 197, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_CONSTANT_NAME, "REPLACE");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_REPLACE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_ADD_AFTER
    CLIST_EDIT_ADD_AFTER = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD AFTER>>", 224, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_CONSTANT_NAME, "ADD_AFTER");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_ADD_AFTER);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_ADD_BEFORE
    CLIST_EDIT_ADD_BEFORE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD BEFORE>", 170, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_CONSTANT_NAME, "ADD_BEFORE");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_ADD_BEFORE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_EDIT_COMMAND
    CLIST_EDIT_EDIT_COMMAND = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "<<EDIT CMD", 282, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_CONSTANT_NAME, "EDIT_COMMAND");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_EDIT_COMMAND);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_DELETE_COMMAND
    CLIST_EDIT_DELETE_COMMAND = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE CMD", 318, 366);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_CONSTANT_NAME, "DELETE_COMMAND");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_DELETE_COMMAND);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_WIDTH, 98);

    // Build control: CLIST_EDIT_LIST_TABLE
    CLIST_EDIT_LIST_TABLE = NewCtrl (hPanel, CTRL_TABLE, "", 81, 468);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CONSTANT_NAME, "LIST_TABLE");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CALLBACK_FUNCTION_POINTER, clistEditButtons);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_SCROLL_BARS, VAL_VERT_SCROLL_BAR);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_SCROLL_BAR_SIZE, VAL_LARGE_SCROLL_BARS);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_TEXT, "");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_MAX_VALUE, HUGE_VAL);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_MIN_VALUE, -HUGE_VAL);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_INCR_VALUE, 1.000000);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_PRECISION, 2);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_WRAP_MODE, VAL_LINE_WRAP);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ENTER_IS_NEWLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_NO_EDIT_TEXT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_UPPER_LEFT_CORNER_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TABLE_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_TABLE_MODE, VAL_COLUMN);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_WIDTH, 22);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ENABLE_ROW_SIZING, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_HEIGHT, 25);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ENABLE_COLUMN_SIZING, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ENABLE_POPUP_MENU, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CELL_TYPE, VAL_CELL_STRING);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CELL_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CELL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_STR_CELL_DFLT_VALUE, "");
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CELL_JUSTIFY, VAL_TOP_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_VERTICAL_GRID_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_HEIGHT, 530);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_WIDTH, 269);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_TOP, 60);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_LEFT, 602);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    InsertTableColumns (hPanel, CLIST_EDIT_LIST_TABLE, -1, 2, VAL_USE_MASTER_CELL_TYPE);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_TEXT, "ID                    COMMAND");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_MAX_VALUE, HUGE_VAL);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_MIN_VALUE, -HUGE_VAL);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_INCR_VALUE, 1.000000);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_PRECISION, 2);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_COLUMN_WIDTH, 184);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_CELL_TYPE, VAL_CELL_STRING);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_CELL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 1, ATTR_VERTICAL_GRID_VISIBLE, 1);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_TEXT, "Repeat");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_MAX_VALUE, -1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_MIN_VALUE, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_INCR_VALUE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_PRECISION, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_COLUMN_WIDTH, 40);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_CELL_TYPE, VAL_CELL_NUMERIC);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_NUM_CELL_DFLT_VALUE, 0);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_CELL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, CLIST_EDIT_LIST_TABLE, 2, ATTR_VERTICAL_GRID_VISIBLE, 1);
    DefaultCtrl (hPanel, CLIST_EDIT_LIST_TABLE);
    SetActiveTableCell (hPanel, CLIST_EDIT_LIST_TABLE, MakePoint (1, 0));
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_VSCROLL_OFFSET, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindTableCellRange (hPanel, CLIST_EDIT_LIST_TABLE, MakeRect(0, 0, 0, 0), "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_SLOT_NUMBER
    CLIST_EDIT_SLOT_NUMBER = NewCtrl (hPanel, CTRL_RECESSED_MENU_RING, "Slot#", 39, 380);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_CONSTANT_NAME, "SLOT_NUMBER");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_DATA_TYPE, VAL_UNSIGNED_CHAR);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 0, "ALL", 0);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 1, "5", 5);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 2, "6", 6);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 3, "7", 7);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 4, "8", 8);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 5, "9", 9);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 6, "10", 10);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 7, "11", 11);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 8, "12", 12);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 9, "13-TIM", 13);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 10, "14", 14);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 11, "15", 15);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 12, "16", 16);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 13, "17", 17);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 14, "18", 18);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 15, "19", 19);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 16, "20", 20);
    InsertListItem (hPanel, CLIST_EDIT_SLOT_NUMBER, 17, "21", 21);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_CTRL_MODE, VAL_NORMAL);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_POINT_SIZE, 14);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_WIDTH, 38);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_HEIGHT, 18);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_COLOR, 0XFFFF99);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_FONT, "Arial");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_POINT_SIZE, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_BGCOLOR, 0X003366);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_SLOT_NUMBER);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_MENU_ARROW_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_TOP, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_LEFT, 393);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: CLIST_EDIT_COMMAND_REPEAT
    CLIST_EDIT_COMMAND_REPEAT = NewCtrl (hPanel, CTRL_NUMERIC, "CmdRepetitions", 38, 132);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_CONSTANT_NAME, "COMMAND_REPEAT");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_WIDTH, 77);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_BGCOLOR, 0XC9D3D0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_CHECK_RANGE, VAL_COERCE);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_COMMAND_REPEAT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_WIDTH, 65);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_TOP, 40);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_LEFT, 55);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_COMMAND_REPEAT, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_COMMAND_COUNT
    CLIST_EDIT_COMMAND_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "CmdCount", 40, 588);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_CONSTANT_NAME, "COMMAND_COUNT");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_FRAME_COLOR, VAL_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_WIDTH, 52);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_BGCOLOR, 0XC9D3D0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_MAX_VALUE, 2147483647);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_COMMAND_COUNT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_WIDTH, 34);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_TOP, 42);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_LEFT, 537);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_COMMAND_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_CONT_IF_ERROR
    CLIST_EDIT_CONT_IF_ERROR = NewCtrl (hPanel, CTRL_CHECK_BOX, "Continue if error?", 39, 298);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_CONSTANT_NAME, "CONT_IF_ERROR");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_WIDTH, 83);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_CONT_IF_ERROR);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_HEIGHT, 18);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_WIDTH, 17);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_TOP, 40);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_LEFT, 214);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: CLIST_EDIT_LAST_FILE
    CLIST_EDIT_LAST_FILE = NewCtrl (hPanel, CTRL_STRING, "LastFile", 61, 469);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_CONSTANT_NAME, "LAST_FILE");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_WIDTH, 39);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, CLIST_EDIT_LAST_FILE);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_WIDTH, 268);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_TOP, 46);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_LEFT, 676);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, CLIST_EDIT_LAST_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_CMD_COMMENT
    CLIST_EDIT_CMD_COMMENT = NewCtrl (hPanel, CTRL_TEXT_BOX, "", 619, 8);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_CONSTANT_NAME, "CMD_COMMENT");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_SCROLL_BARS, VAL_BOTH_SCROLL_BARS);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_SCROLL_BAR_SIZE, VAL_SMALL_SCROLL_BARS);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, CLIST_EDIT_CMD_COMMENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_MAX_ENTRY_LENGTH, 600);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_ENTER_IS_NEWLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_FIRST_VISIBLE_LINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_EXTRA_LINES, -1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_HEIGHT, 53);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_WIDTH, 353);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_TOP, 636);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_LEFT, 363);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, CLIST_EDIT_CMD_COMMENT, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_CLIST_COMMENT
    CLIST_EDIT_CLIST_COMMENT = NewCtrl (hPanel, CTRL_TEXT_BOX, "", 618, 468);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_CONSTANT_NAME, "CLIST_COMMENT");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_SCROLL_BARS, VAL_BOTH_SCROLL_BARS);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_SCROLL_BAR_SIZE, VAL_SMALL_SCROLL_BARS);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, CLIST_EDIT_CLIST_COMMENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_MAX_ENTRY_LENGTH, 600);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_ENTER_IS_NEWLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_FIRST_VISIBLE_LINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_EXTRA_LINES, -1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_HEIGHT, 53);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_WIDTH, 269);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_TOP, 627);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_LEFT, 404);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, CLIST_EDIT_CLIST_COMMENT, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_CLIST_REP
    CLIST_EDIT_CLIST_REP = NewCtrl (hPanel, CTRL_NUMERIC, "Repetitions", 86, 380);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_CONSTANT_NAME, "CLIST_REP");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_WIDTH, 56);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_CHECK_RANGE, VAL_IGNORE);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, CLIST_EDIT_CLIST_REP);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_TOP, 72);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_LEFT, 388);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_CLIST_REP, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_DECORATION_3
    CLIST_EDIT_DECORATION_3 = NewCtrl (hPanel, CTRL_RAISED_BOX, "", 23, 365);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_CONSTANT_NAME, "DECORATION_3");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_FRAME_COLOR, 0XC9D3D0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_HEIGHT, 120);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_WIDTH, 102);

    // Build control: CLIST_EDIT_TEXTMSG_3
    CLIST_EDIT_TEXTMSG_3 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 64, 101);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_CONSTANT_NAME, "TEXTMSG_3");
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_VISIBLE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_COLOR, VAL_RED);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_DFLT_VALUE, "RING FILLED IN fillUirControls.c");
    DefaultCtrl (hPanel, CLIST_EDIT_TEXTMSG_3);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_WIDTH, 155);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_TEXTMSG_3, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_TEXTMSG_4
    CLIST_EDIT_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 2, 130);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_DFLT_VALUE, " COMMAND ");
    DefaultCtrl (hPanel, CLIST_EDIT_TEXTMSG_4);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_WIDTH, 103);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_TEXTMSG
    CLIST_EDIT_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 2, 534);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_DFLT_VALUE, " COMMAND LIST ");
    DefaultCtrl (hPanel, CLIST_EDIT_TEXTMSG);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_WIDTH, 146);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_TEXTMSG_5
    CLIST_EDIT_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 17, 120);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_DFLT_VALUE, " COMMAND ");
    DefaultCtrl (hPanel, CLIST_EDIT_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_WIDTH, 103);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: CLIST_EDIT_TEXTMSG_6
    CLIST_EDIT_TEXTMSG_6 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 17, 534);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_CONSTANT_NAME, "TEXTMSG_6");
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_DFLT_VALUE, " COMMAND LIST ");
    DefaultCtrl (hPanel, CLIST_EDIT_TEXTMSG_6);
//  SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_WIDTH, 146);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, CLIST_EDIT_TEXTMSG_6, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_5, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_6, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_4, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG_3, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_ZPLANE_POSITION, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_ZPLANE_POSITION, 17);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_ZPLANE_POSITION, 18);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_ZPLANE_POSITION, 19);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_ZPLANE_POSITION, 20);
    SetCtrlAttribute (hPanel, CLIST_EDIT_TEXTMSG, ATTR_ZPLANE_POSITION, 21);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_ZPLANE_POSITION, 22);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DECORATION_3, ATTR_ZPLANE_POSITION, 23);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_ZPLANE_POSITION, 24);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_ZPLANE_POSITION, 25);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_ZPLANE_POSITION, 26);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_ZPLANE_POSITION, 27);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_RING, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_LIST, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CLCOMM, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLEAR_CCOMM, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SAVE_LIST, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LOAD_LIST, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, CLIST_EDIT_INSERT_LIST, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SEND_CLIST_TO_ROD, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, CLIST_EDIT_REPLACE, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_AFTER, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, CLIST_EDIT_ADD_BEFORE, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, CLIST_EDIT_EDIT_COMMAND, ATTR_CTRL_TAB_POSITION, 11);
    SetCtrlAttribute (hPanel, CLIST_EDIT_DELETE_COMMAND, ATTR_CTRL_TAB_POSITION, 12);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LIST_TABLE, ATTR_CTRL_TAB_POSITION, 13);
    SetCtrlAttribute (hPanel, CLIST_EDIT_SLOT_NUMBER, ATTR_CTRL_TAB_POSITION, 14);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_REPEAT, ATTR_CTRL_TAB_POSITION, 15);
    SetCtrlAttribute (hPanel, CLIST_EDIT_COMMAND_COUNT, ATTR_CTRL_TAB_POSITION, 16);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CONT_IF_ERROR, ATTR_CTRL_TAB_POSITION, 17);
    SetCtrlAttribute (hPanel, CLIST_EDIT_LAST_FILE, ATTR_CTRL_TAB_POSITION, 18);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CMD_COMMENT, ATTR_CTRL_TAB_POSITION, 19);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_COMMENT, ATTR_CTRL_TAB_POSITION, 20);
    SetCtrlAttribute (hPanel, CLIST_EDIT_CLIST_REP, ATTR_CTRL_TAB_POSITION, 21);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(676*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(745*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(29*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(13*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_TOP, VAL_AUTO_CENTER);
    SetPanelAttribute (hPanel, ATTR_LEFT, VAL_AUTO_CENTER);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    buildMenu_cmdListEditor (hPanel);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildMenu_cmdListEditor ():  Build UI object CLIST_MENU; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildMenu_cmdListEditor (int hParentPanel)
{
    int hMenubar;
    int error = 0;

    // Create the menubar
    hMenubar = NewMenuBar (hParentPanel);
    SetMenuBarAttribute (hMenubar, 0, ATTR_CONSTANT_NAME, "CLIST_MENU");

    // Build menu: CLIST_MENU_GOTO_MAIN
    CLIST_MENU_GOTO_MAIN = NewMenu (hMenubar, "Main", -1);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_GOTO_MAIN, ATTR_CONSTANT_NAME, "GOTO_MAIN");
//  SetMenuBarAttribute (hMenubar, CLIST_MENU_GOTO_MAIN, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_GOTO_MAIN, ATTR_CALLBACK_FUNCTION_POINTER, commandListEditMenu);

    // Build menu: CLIST_MENU_PRIM_LIST_EDIT
    CLIST_MENU_PRIM_LIST_EDIT = NewMenu (hMenubar, "PrimListEditor", -1);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_PRIM_LIST_EDIT, ATTR_CONSTANT_NAME, "PRIM_LIST_EDIT");
//  SetMenuBarAttribute (hMenubar, CLIST_MENU_PRIM_LIST_EDIT, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_PRIM_LIST_EDIT, ATTR_CALLBACK_FUNCTION_POINTER, commandListEditMenu);

    // Build menu: CLIST_MENU_DATA_FILE_CREATOR
    CLIST_MENU_DATA_FILE_CREATOR = NewMenu (hMenubar, "DataFileCreator", -1);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_DATA_FILE_CREATOR, ATTR_CONSTANT_NAME, "DATA_FILE_CREATOR");
//  SetMenuBarAttribute (hMenubar, CLIST_MENU_DATA_FILE_CREATOR, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_DATA_FILE_CREATOR, ATTR_CALLBACK_FUNCTION_POINTER, commandListEditMenu);

    // Build menu: CLIST_MENU_CLOSE
    CLIST_MENU_CLOSE = NewMenu (hMenubar, "HideWindow", -1);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_CLOSE, ATTR_CONSTANT_NAME, "CLOSE");
//  SetMenuBarAttribute (hMenubar, CLIST_MENU_CLOSE, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, CLIST_MENU_CLOSE, ATTR_CALLBACK_FUNCTION_POINTER, commandListEditMenu);

    //We're done!
    return hMenubar;
Error:
    return error;
}
