
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------
//
//  "uiCmdList.c"
//
//  Implementation file for UI build functions
//
//  Date: 11-07-2004
//
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//  Include header files
//----------------------------------------------------------------------------------------------------

#include "uiCmdList.h"
#include <Utility.h> // definition of NULL
#include "toolbox.h" // definition of errChk macro

extern int CVIFUNC_C GetUserBitmapFromText (char *imageText, int version, int *bitmapId); // needed if there are bitmaps
extern int CVIFUNC_C NewActiveXCtrlFromPersistence (int panel, const char *ctrlLabel,
                                                    int top, int left, char *defaultIID,
                                                    int binaryFormat, char *persistenceText,
                                                    HRESULT *activeXError); // needed if there are ActiveX controls

//----------------------------------------------------------------------------------------------------
//  Prototype UI build functions.
//----------------------------------------------------------------------------------------------------

int buildPanel_bocSetup (int hParentPanel);
int buildPanel_busTest (int hParentPanel);
int buildPanel_compareFiles (int hParentPanel);
int buildPanel_delayCmd (int hParentPanel);
int buildPanel_runChildCmdList (int hParentPanel);
int buildPanel_runExternalProg (int hParentPanel);
int buildPanel_rwFpgaFlash (int hParentPanel);
int buildPanel_rwMdsp (int hParentPanel);
int buildPanel_rwSdsp (int hParentPanel);
int buildPanel_rwVme (int hParentPanel);
int buildPanel_sendPrimList (int hParentPanel);
int buildPanel_wMdspFlash (int hParentPanel);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: BOC_SETUP
// ------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: BUS_TEST
// ------------------------------------------------------------------------------------------------

// Control: BUS_TEST_REG_ADDR_FILE
int CVICALLBACK busTest (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: BUS_TEST_ONE_REG
int CVICALLBACK busTest (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: BUS_TEST_ADDR_TEST_OPTION
int CVICALLBACK busTest (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: COMP_FILES
// ------------------------------------------------------------------------------------------------

// Control: COMP_FILES_ROD_FILE
int CVICALLBACK compFiles (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: COMP_FILES_COMMON_FILE
int CVICALLBACK compFiles (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: COMP_FILES_WORD_SIZE
int CVICALLBACK compFiles (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: DELAY_CMDP
// ------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RUN_CLIST
// ------------------------------------------------------------------------------------------------

// Control: RUN_CLIST_FILE
int CVICALLBACK runChildCommandList (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RUN_EXTPRG
// ------------------------------------------------------------------------------------------------

// Control: RUN_EXTPRG_ROD_FILE
int CVICALLBACK runExtProgram (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RUN_EXTPRG_EXT_PRG
int CVICALLBACK runExtProgram (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RUN_EXTPRG_DATA_FILE_OPT
int CVICALLBACK runExtProgram (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RW_FPGAF
// ------------------------------------------------------------------------------------------------

// Control: RW_FPGAF_OUT_FILE
int CVICALLBACK RWFpgaFlash (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_FPGAF_INP_FILE
int CVICALLBACK RWFpgaFlash (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_FPGAF_READ_NOT_WRITE
int CVICALLBACK RWFpgaFlash (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_FPGAF_START_ADDRESS
int CVICALLBACK RWFpgaFlash (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RW_MASTERP
// ------------------------------------------------------------------------------------------------

// Control: RW_MASTERP_OUT_FILE
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_INP_FILE
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_START_ADDR
int CVICALLBACK wordBoundary (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_LENGTH
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_COMP_OPTION
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_OUTPUT_OPT
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_INP_OPTION
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_READ_NOT_WRITE
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_MASTERP_BLOCK_ACCESS
int CVICALLBACK RWMaster (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RW_SLAVEP
// ------------------------------------------------------------------------------------------------

// Control: RW_SLAVEP_OUT_FILE
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_INP_FILE
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_START_ADDR
int CVICALLBACK wordBoundary (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_LENGTH
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_COMP_OPTION
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_OUTPUT_OPT
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_INP_OPTION
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_SLAVEP_READ_NOT_WRITE
int CVICALLBACK RWSlave (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: RW_VMEP
// ------------------------------------------------------------------------------------------------

// Control: RW_VMEP_OUT_FILE
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_INP_FILE
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_LENGTH
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_COMP_OPTION
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_OUTPUT_OPT
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_INP_OPTION
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_READ_NOT_WRITE
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: RW_VMEP_VME_ACCESS
int CVICALLBACK RWVme (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: SEND_PLIST
// ------------------------------------------------------------------------------------------------

// Control: SEND_PLIST_INP_FILE
int CVICALLBACK sendPrimList (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SEND_PLIST_EDIT_PRIM_LIST
int CVICALLBACK sendPrimList (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: W_MDSPF
// ------------------------------------------------------------------------------------------------

// Control: W_MDSPF_INP_FILE
int CVICALLBACK WMDSPFlash (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);


//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  BOC_SETUP
// ------------------------------------------------------------------------------------------------

int BOC_SETUP_DECORATION                     = 0;  // control identifier
int BOC_SETUP_TEXTMSG_4                      = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  BUS_TEST
// ------------------------------------------------------------------------------------------------

int BUS_TEST_DATA_WIDTH                      = 0;  // control identifier
int BUS_TEST_REG_ADDR                        = 0;  // control identifier
int BUS_TEST_ADDR_REG                        = 0;  // control identifier
int BUS_TEST_REG_ADDR_FILE                   = 0;  // control identifier
int BUS_TEST_ONE_REG                         = 0;  // control identifier
int BUS_TEST_ADDR_TEST_OPTION                = 0;  // control identifier
int BUS_TEST_FLOAT_ONE                       = 0;  // control identifier
int BUS_TEST_DECORATION                      = 0;  // control identifier
int BUS_TEST_TEXTMSG_4                       = 0;  // control identifier
int BUS_TEST_TEXTMSG_2                       = 0;  // control identifier
int BUS_TEST_TEXTMSG                         = 0;  // control identifier
int BUS_TEST_TEXTMSG_3                       = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  COMP_FILES
// ------------------------------------------------------------------------------------------------

int COMP_FILES_DATA_MASK0                    = 0;  // control identifier
int COMP_FILES_ROD_FILE                      = 0;  // control identifier
int COMP_FILES_COMMON_FILE                   = 0;  // control identifier
int COMP_FILES_WORD_SIZE                     = 0;  // control identifier
int COMP_FILES_DATA_MASK1                    = 0;  // control identifier
int COMP_FILES_DECORATION                    = 0;  // control identifier
int COMP_FILES_TEXTMSG_4                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  DELAY_CMDP
// ------------------------------------------------------------------------------------------------

int DELAY_CMDP_DELAY                         = 0;  // control identifier
int DELAY_CMDP_DECORATION                    = 0;  // control identifier
int DELAY_CMDP_TEXTMSG_4                     = 0;  // control identifier
int DELAY_CMDP_TEXTMSG                       = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RUN_CLIST
// ------------------------------------------------------------------------------------------------

int RUN_CLIST_FILE                           = 0;  // control identifier
int RUN_CLIST_DECORATION                     = 0;  // control identifier
int RUN_CLIST_TEXTMSG_4                      = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RUN_EXTPRG
// ------------------------------------------------------------------------------------------------

int RUN_EXTPRG_TIMEOUT                       = 0;  // control identifier
int RUN_EXTPRG_ROD_FILE                      = 0;  // control identifier
int RUN_EXTPRG_EXT_PRG                       = 0;  // control identifier
int RUN_EXTPRG_DATA_FILE_OPT                 = 0;  // control identifier
int RUN_EXTPRG_WAIT_FOR_TERM                 = 0;  // control identifier
int RUN_EXTPRG_DECORATION                    = 0;  // control identifier
int RUN_EXTPRG_TEXTMSG_4                     = 0;  // control identifier
int RUN_EXTPRG_TEXTMSG                       = 0;  // control identifier
int RUN_EXTPRG_DECORATION_2                  = 0;  // control identifier
int RUN_EXTPRG_TEXTMSG_2                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RW_FPGAF
// ------------------------------------------------------------------------------------------------

int RW_FPGAF_OUT_FILE                        = 0;  // control identifier
int RW_FPGAF_INP_FILE                        = 0;  // control identifier
int RW_FPGAF_SIZE                            = 0;  // control identifier
int RW_FPGAF_READ_NOT_WRITE                  = 0;  // control identifier
int RW_FPGAF_START_ADDRESS                   = 0;  // control identifier
int RW_FPGAF_DECORATION                      = 0;  // control identifier
int RW_FPGAF_TEXTMSG_4                       = 0;  // control identifier
int RW_FPGAF_DECORATION_2                    = 0;  // control identifier
int RW_FPGAF_TEXTMSG                         = 0;  // control identifier
int RW_FPGAF_TEXTMSG_5                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_2                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_3                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_6                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_7                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_8                       = 0;  // control identifier
int RW_FPGAF_TEXTMSG_12                      = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RW_MASTERP
// ------------------------------------------------------------------------------------------------

int RW_MASTERP_OUT_FILE                      = 0;  // control identifier
int RW_MASTERP_INP_FILE                      = 0;  // control identifier
int RW_MASTERP_DATA_MASK                     = 0;  // control identifier
int RW_MASTERP_COMP_DATA                     = 0;  // control identifier
int RW_MASTERP_INPUT_DATA                    = 0;  // control identifier
int RW_MASTERP_START_ADDR                    = 0;  // control identifier
int RW_MASTERP_DELAY                         = 0;  // control identifier
int RW_MASTERP_LENGTH                        = 0;  // control identifier
int RW_MASTERP_COMP_OPTION                   = 0;  // control identifier
int RW_MASTERP_OUTPUT_OPT                    = 0;  // control identifier
int RW_MASTERP_INP_OPTION                    = 0;  // control identifier
int RW_MASTERP_READ_NOT_WRITE                = 0;  // control identifier
int RW_MASTERP_BLOCK_ACCESS                  = 0;  // control identifier
int RW_MASTERP_WRITE_OR_APPEND               = 0;  // control identifier
int RW_MASTERP_DECORATION                    = 0;  // control identifier
int RW_MASTERP_TEXTMSG_4                     = 0;  // control identifier
int RW_MASTERP_DECORATION_2                  = 0;  // control identifier
int RW_MASTERP_TEXTMSG                       = 0;  // control identifier
int RW_MASTERP_TEXTMSG_5                     = 0;  // control identifier
int RW_MASTERP_TEXTMSG_12                    = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RW_SLAVEP
// ------------------------------------------------------------------------------------------------

int RW_SLAVEP_OUT_FILE                       = 0;  // control identifier
int RW_SLAVEP_INP_FILE                       = 0;  // control identifier
int RW_SLAVEP_COMP_DATA                      = 0;  // control identifier
int RW_SLAVEP_INPUT_DATA                     = 0;  // control identifier
int RW_SLAVEP_START_ADDR                     = 0;  // control identifier
int RW_SLAVEP_LENGTH                         = 0;  // control identifier
int RW_SLAVEP_COMP_OPTION                    = 0;  // control identifier
int RW_SLAVEP_OUTPUT_OPT                     = 0;  // control identifier
int RW_SLAVEP_INP_OPTION                     = 0;  // control identifier
int RW_SLAVEP_READ_NOT_WRITE                 = 0;  // control identifier
int RW_SLAVEP_WRITE_OR_APPEND                = 0;  // control identifier
int RW_SLAVEP_SLAVE_NUMBER                   = 0;  // control identifier
int RW_SLAVEP_DECORATION                     = 0;  // control identifier
int RW_SLAVEP_TEXTMSG_4                      = 0;  // control identifier
int RW_SLAVEP_TEXTMSG                        = 0;  // control identifier
int RW_SLAVEP_TEXTMSG_5                      = 0;  // control identifier
int RW_SLAVEP_DECORATION_3                   = 0;  // control identifier
int RW_SLAVEP_TEXTMSG_12                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  RW_VMEP
// ------------------------------------------------------------------------------------------------

int RW_VMEP_OUT_FILE                         = 0;  // control identifier
int RW_VMEP_INP_FILE                         = 0;  // control identifier
int RW_VMEP_COMP_DATA                        = 0;  // control identifier
int RW_VMEP_INPUT_DATA                       = 0;  // control identifier
int RW_VMEP_START_ADDR                       = 0;  // control identifier
int RW_VMEP_LENGTH                           = 0;  // control identifier
int RW_VMEP_COMP_OPTION                      = 0;  // control identifier
int RW_VMEP_OUTPUT_OPT                       = 0;  // control identifier
int RW_VMEP_INP_OPTION                       = 0;  // control identifier
int RW_VMEP_NO_BLOCK_ACCESS                  = 0;  // control identifier
int RW_VMEP_READ_NOT_WRITE                   = 0;  // control identifier
int RW_VMEP_WRITE_OR_APPEND                  = 0;  // control identifier
int RW_VMEP_VME_ACCESS                       = 0;  // control identifier
int RW_VMEP_DECORATION                       = 0;  // control identifier
int RW_VMEP_TEXTMSG_4                        = 0;  // control identifier
int RW_VMEP_TEXTMSG                          = 0;  // control identifier
int RW_VMEP_TEXTMSG_5                        = 0;  // control identifier
int RW_VMEP_DECORATION_3                     = 0;  // control identifier
int RW_VMEP_TEXTMSG_2                        = 0;  // control identifier
int RW_VMEP_TEXTMSG_6                        = 0;  // control identifier
int RW_VMEP_TEXTMSG_7                        = 0;  // control identifier
int RW_VMEP_TEXTMSG_12                       = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  SEND_PLIST
// ------------------------------------------------------------------------------------------------

int SEND_PLIST_INP_FILE                      = 0;  // control identifier
int SEND_PLIST_REPEAT_BUILD_EXEC             = 0;  // control identifier
int SEND_PLIST_EDIT_PRIM_LIST                = 0;  // control identifier
int SEND_PLIST_DECORATION                    = 0;  // control identifier
int SEND_PLIST_TEXTMSG_4                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  W_MDSPF
// ------------------------------------------------------------------------------------------------

int W_MDSPF_INP_FILE                         = 0;  // control identifier
int W_MDSPF_SIZE                             = 0;  // control identifier
int W_MDSPF_START_ADDRESS                    = 0;  // control identifier
int W_MDSPF_DECORATION                       = 0;  // control identifier
int W_MDSPF_TEXTMSG_4                        = 0;  // control identifier
int W_MDSPF_TEXTMSG_5                        = 0;  // control identifier
int W_MDSPF_TEXTMSG_2                        = 0;  // control identifier
int W_MDSPF_TEXTMSG_3                        = 0;  // control identifier
int W_MDSPF_TEXTMSG_6                        = 0;  // control identifier

//----------------------------------------------------------------------------------------------------
// buildPanel_bocSetup ():  Build UI object BOC_SETUP; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_bocSetup (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "BOCSetup", 61, 87, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "BOC_SETUP");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: BOC_SETUP_DECORATION
    BOC_SETUP_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_HEIGHT, 160);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_WIDTH, 342);

    // Build control: BOC_SETUP_TEXTMSG_4
    BOC_SETUP_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, BOC_SETUP_TEXTMSG_4);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, BOC_SETUP_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, BOC_SETUP_TEXTMSG_4, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, BOC_SETUP_DECORATION, ATTR_ZPLANE_POSITION, 1);

    // Set up TAB order if necessary

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(61*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(87*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_busTest ():  Build UI object BUS_TEST; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_busTest (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "BusTest", 21, 47, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "BUS_TEST");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: BUS_TEST_DATA_WIDTH
    BUS_TEST_DATA_WIDTH = NewCtrl (hPanel, CTRL_NUMERIC, "DataWidth", 72, 150);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_CONSTANT_NAME, "DATA_WIDTH");
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_WIDTH, 63);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_MAX_VALUE, 32);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_MIN_VALUE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_CHECK_RANGE, VAL_COERCE);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_DFLT_VALUE, 32);
    DefaultCtrl (hPanel, BUS_TEST_DATA_WIDTH);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_WIDTH, 48);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_TOP, 57);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_LEFT, 142);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_DATA_WIDTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_REG_ADDR
    BUS_TEST_REG_ADDR = NewCtrl (hPanel, CTRL_NUMERIC, "TestRegAddress", 159, 142);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_CONSTANT_NAME, "REG_ADDR");
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_WIDTH, 96);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, BUS_TEST_REG_ADDR);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_TOP, 144);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_LEFT, 131);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_REG_ADDR, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_ADDR_REG
    BUS_TEST_ADDR_REG = NewCtrl (hPanel, CTRL_NUMERIC, "AddrRegAddress", 245, 228);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_CONSTANT_NAME, "ADDR_REG");
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_WIDTH, 97);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, BUS_TEST_ADDR_REG);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_TOP, 230);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_LEFT, 217);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_ADDR_REG, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_REG_ADDR_FILE
    BUS_TEST_REG_ADDR_FILE = NewCtrl (hPanel, CTRL_STRING, "TEST REGS MDSP ADDRESSES", 200, 10);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_CONSTANT_NAME, "REG_ADDR_FILE");
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_CALLBACK_FUNCTION_POINTER, busTest);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_WIDTH, 167);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, BUS_TEST_REG_ADDR_FILE);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_TOP, 186);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_LEFT, 13);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, BUS_TEST_REG_ADDR_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_ONE_REG
    BUS_TEST_ONE_REG = NewCtrl (hPanel, CTRL_VSWITCH, "", 105, 142);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_CONSTANT_NAME, "ONE_REG");
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_CALLBACK_FUNCTION_POINTER, busTest);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_BGCOLOR, 0XCACACA);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, BUS_TEST_ONE_REG);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_ON_TEXT, "ONE REGISTER");
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_OFF_TEXT, "SET OF REGISTERS");
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_HEIGHT, 25);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_WIDTH, 14);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_TOP, 84);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_LEFT, 122);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: BUS_TEST_ADDR_TEST_OPTION
    BUS_TEST_ADDR_TEST_OPTION = NewCtrl (hPanel, CTRL_VSWITCH, "", 241, 180);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_CONSTANT_NAME, "ADDR_TEST_OPTION");
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, busTest);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_BGCOLOR, 0XCACACA);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_TEXT_BOLD, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, BUS_TEST_ADDR_TEST_OPTION);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_ON_TEXT, "On");
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_OFF_TEXT, "Off");
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_HEIGHT, 25);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_WIDTH, 14);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_TOP, 220);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_LEFT, 160);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: BUS_TEST_FLOAT_ONE
    BUS_TEST_FLOAT_ONE = NewCtrl (hPanel, CTRL_CHECK_BOX, "Floating 1 Test Only", 278, 140);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_CONSTANT_NAME, "FLOAT_ONE");
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_WIDTH, 97);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, BUS_TEST_FLOAT_ONE);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_TOP, 278);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_LEFT, 156);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: BUS_TEST_DECORATION
    BUS_TEST_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_HEIGHT, 280);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_WIDTH, 342);

    // Build control: BUS_TEST_TEXTMSG_4
    BUS_TEST_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, BUS_TEST_TEXTMSG_4);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_TEXTMSG_2
    BUS_TEST_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 245, 17);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_DFLT_VALUE, "AddrLatchRegister on write?");
    DefaultCtrl (hPanel, BUS_TEST_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_WIDTH, 164);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_TEXTMSG
    BUS_TEST_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 74, 199);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_DFLT_VALUE, "bits");
    DefaultCtrl (hPanel, BUS_TEST_TEXTMSG);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_WIDTH, 19);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: BUS_TEST_TEXTMSG_3
    BUS_TEST_TEXTMSG_3 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 347, 102);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_CONSTANT_NAME, "TEXTMSG_3");
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_BGCOLOR, 0XCACACA);
    dataPtr = 
        "Test sequence:\n1. Floating 1 test\n2. Const. data test { 0, F, 5, A }\n3. Rando"
        "m data";
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_DFLT_VALUE, dataPtr);
    DefaultCtrl (hPanel, BUS_TEST_TEXTMSG_3);
//  SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_HEIGHT, 54);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_WIDTH, 149);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, BUS_TEST_TEXTMSG_3, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_3, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_2, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, BUS_TEST_TEXTMSG_4, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, BUS_TEST_DECORATION, ATTR_ZPLANE_POSITION, 11);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, BUS_TEST_DATA_WIDTH, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_REG, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, BUS_TEST_REG_ADDR_FILE, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, BUS_TEST_ONE_REG, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, BUS_TEST_ADDR_TEST_OPTION, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, BUS_TEST_FLOAT_ONE, ATTR_CTRL_TAB_POSITION, 6);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(21*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(47*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_compareFiles ():  Build UI object COMP_FILES; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_compareFiles (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "CompareFiles", 41, 67, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "COMP_FILES");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: COMP_FILES_DATA_MASK0
    COMP_FILES_DATA_MASK0 = NewCtrl (hPanel, CTRL_NUMERIC, "DataMask", 82, 215);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_CONSTANT_NAME, "DATA_MASK0");
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_WIDTH, 60);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_DFLT_VALUE, -1);
    DefaultCtrl (hPanel, COMP_FILES_DATA_MASK0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_TOP, 66);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_LEFT, 190);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, COMP_FILES_DATA_MASK0, "", VAL_DS_READ, NULL, NULL);

    // Build control: COMP_FILES_ROD_FILE
    COMP_FILES_ROD_FILE = NewCtrl (hPanel, CTRL_STRING, "ROD DATA FILE (in RodData directory)", 196, 10);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_CONSTANT_NAME, "ROD_FILE");
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_CALLBACK_FUNCTION_POINTER, compFiles);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_WIDTH, 190);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, COMP_FILES_ROD_FILE);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_TOP, 182);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, COMP_FILES_ROD_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: COMP_FILES_COMMON_FILE
    COMP_FILES_COMMON_FILE = NewCtrl (hPanel, CTRL_STRING, "COMMON DATA FILE", 151, 10);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_CONSTANT_NAME, "COMMON_FILE");
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_CALLBACK_FUNCTION_POINTER, compFiles);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_WIDTH, 109);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, COMP_FILES_COMMON_FILE);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_TOP, 137);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_LEFT, 12);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, COMP_FILES_COMMON_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: COMP_FILES_WORD_SIZE
    COMP_FILES_WORD_SIZE = NewCtrl (hPanel, CTRL_RECESSED_MENU_RING, "WordSize", 82, 61);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_CONSTANT_NAME, "WORD_SIZE");
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_CALLBACK_FUNCTION_POINTER, compFiles);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_DATA_TYPE, VAL_INTEGER);
    InsertListItem (hPanel, COMP_FILES_WORD_SIZE, 0, "8 bits", 1);
    InsertListItem (hPanel, COMP_FILES_WORD_SIZE, 1, "16 bits", 2);
    InsertListItem (hPanel, COMP_FILES_WORD_SIZE, 2, "32 bits", 4);
    InsertListItem (hPanel, COMP_FILES_WORD_SIZE, 3, "48 bits", 6);
    InsertListItem (hPanel, COMP_FILES_WORD_SIZE, 4, "64 bits", 8);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_WIDTH, 57);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_DFLT_INDEX, 2);
    DefaultCtrl (hPanel, COMP_FILES_WORD_SIZE);
//  SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_MENU_ARROW_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_WIDTH, 59);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_TOP, 67);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_LEFT, 62);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: COMP_FILES_DATA_MASK1
    COMP_FILES_DATA_MASK1 = NewCtrl (hPanel, CTRL_NUMERIC, "", 82, 145);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_CONSTANT_NAME, "DATA_MASK1");
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_BGCOLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_DFLT_VALUE, -1);
    DefaultCtrl (hPanel, COMP_FILES_DATA_MASK1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_TOP, 66);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_LEFT, 153);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, COMP_FILES_DATA_MASK1, "", VAL_DS_READ, NULL, NULL);

    // Build control: COMP_FILES_DECORATION
    COMP_FILES_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_HEIGHT, 260);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_WIDTH, 342);

    // Build control: COMP_FILES_TEXTMSG_4
    COMP_FILES_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, COMP_FILES_TEXTMSG_4);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, COMP_FILES_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, COMP_FILES_TEXTMSG_4, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, COMP_FILES_DECORATION, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_ZPLANE_POSITION, 6);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK0, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, COMP_FILES_ROD_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, COMP_FILES_COMMON_FILE, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, COMP_FILES_WORD_SIZE, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, COMP_FILES_DATA_MASK1, ATTR_CTRL_TAB_POSITION, 4);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(41*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(67*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_delayCmd ():  Build UI object DELAY_CMDP; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_delayCmd (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "Delay", 61, 87, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "DELAY_CMDP");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: DELAY_CMDP_DELAY
    DELAY_CMDP_DELAY = NewCtrl (hPanel, CTRL_NUMERIC, "Delay", 99, 122);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_CONSTANT_NAME, "DELAY");
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_DATA_TYPE, VAL_FLOAT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_WIDTH, 35);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_MAX_VALUE, 1000.000000);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_MIN_VALUE, 0.000000);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_INCR_VALUE, 1.000000);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_PRECISION, 3);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_CHECK_RANGE, VAL_COERCE);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_DFLT_VALUE, 1.000000);
    DefaultCtrl (hPanel, DELAY_CMDP_DELAY);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_WIDTH, 74);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_TOP, 84);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_LEFT, 146);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, DELAY_CMDP_DELAY, "", VAL_DS_READ, NULL, NULL);

    // Build control: DELAY_CMDP_DECORATION
    DELAY_CMDP_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_HEIGHT, 160);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_WIDTH, 342);

    // Build control: DELAY_CMDP_TEXTMSG_4
    DELAY_CMDP_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, DELAY_CMDP_TEXTMSG_4);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, DELAY_CMDP_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: DELAY_CMDP_TEXTMSG
    DELAY_CMDP_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 101, 197);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_DFLT_VALUE, "sec");
    DefaultCtrl (hPanel, DELAY_CMDP_TEXTMSG);
//  SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_WIDTH, 20);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, DELAY_CMDP_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, DELAY_CMDP_TEXTMSG_4, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, DELAY_CMDP_DECORATION, ATTR_ZPLANE_POSITION, 3);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, DELAY_CMDP_DELAY, ATTR_CTRL_TAB_POSITION, 0);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(61*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(87*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_runChildCmdList ():  Build UI object RUN_CLIST; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_runChildCmdList (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RunChildCommandList", 51, 77, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RUN_CLIST");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RUN_CLIST_FILE
    RUN_CLIST_FILE = NewCtrl (hPanel, CTRL_STRING, "Child Command List", 101, 10);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_CONSTANT_NAME, "FILE");
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_CALLBACK_FUNCTION_POINTER, runChildCommandList);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_WIDTH, 113);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RUN_CLIST_FILE);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_TOP, 87);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_LEFT, 126);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RUN_CLIST_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_CLIST_DECORATION
    RUN_CLIST_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_HEIGHT, 160);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_WIDTH, 342);

    // Build control: RUN_CLIST_TEXTMSG_4
    RUN_CLIST_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RUN_CLIST_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RUN_CLIST_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RUN_CLIST_TEXTMSG_4, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RUN_CLIST_DECORATION, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_ZPLANE_POSITION, 2);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RUN_CLIST_FILE, ATTR_CTRL_TAB_POSITION, 0);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(51*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(77*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_runExternalProg ():  Build UI object RUN_EXTPRG; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_runExternalProg (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RunExternalProgram", 51, 77, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RUN_EXTPRG");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RUN_EXTPRG_TIMEOUT
    RUN_EXTPRG_TIMEOUT = NewCtrl (hPanel, CTRL_NUMERIC, "Timeout", 247, 126);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_CONSTANT_NAME, "TIMEOUT");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_WIDTH, 48);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_MAX_VALUE, 1000);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_CHECK_RANGE, VAL_COERCE);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_DFLT_VALUE, 1000);
    DefaultCtrl (hPanel, RUN_EXTPRG_TIMEOUT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_WIDTH, 71);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_TOP, 232);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_LEFT, 143);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RUN_EXTPRG_TIMEOUT, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_EXTPRG_ROD_FILE
    RUN_EXTPRG_ROD_FILE = NewCtrl (hPanel, CTRL_STRING, "", 176, 10);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_CONSTANT_NAME, "ROD_FILE");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_CALLBACK_FUNCTION_POINTER, runExtProgram);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RUN_EXTPRG_ROD_FILE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_TOP, 162);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_LEFT, 104);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RUN_EXTPRG_ROD_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_EXTPRG_EXT_PRG
    RUN_EXTPRG_EXT_PRG = NewCtrl (hPanel, CTRL_STRING, "EXTERNAL PROGRAM(+options)", 131, 10);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_CONSTANT_NAME, "EXT_PRG");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_CALLBACK_FUNCTION_POINTER, runExtProgram);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_WIDTH, 164);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RUN_EXTPRG_EXT_PRG);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_TOP, 117);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_LEFT, 13);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RUN_EXTPRG_EXT_PRG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_EXTPRG_DATA_FILE_OPT
    RUN_EXTPRG_DATA_FILE_OPT = NewCtrl (hPanel, CTRL_CHECK_BOX, "ROD DATA FILE (in RodData directory)", 159, 11);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_CONSTANT_NAME, "DATA_FILE_OPT");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_CALLBACK_FUNCTION_POINTER, runExtProgram);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_WIDTH, 190);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RUN_EXTPRG_DATA_FILE_OPT);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_HEIGHT, 17);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_TOP, 160);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_LEFT, 28);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RUN_EXTPRG_WAIT_FOR_TERM
    RUN_EXTPRG_WAIT_FOR_TERM = NewCtrl (hPanel, CTRL_CHECK_BOX, "Wait for program termination?", 209, 86);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_CONSTANT_NAME, "WAIT_FOR_TERM");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_WIDTH, 170);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_DFLT_VALUE, 1);
    DefaultCtrl (hPanel, RUN_EXTPRG_WAIT_FOR_TERM);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_HEIGHT, 17);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_TOP, 210);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_LEFT, 104);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RUN_EXTPRG_DECORATION
    RUN_EXTPRG_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_HEIGHT, 260);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_WIDTH, 342);

    // Build control: RUN_EXTPRG_TEXTMSG_4
    RUN_EXTPRG_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RUN_EXTPRG_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RUN_EXTPRG_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_EXTPRG_TEXTMSG
    RUN_EXTPRG_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 59, 73);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_DFLT_VALUE, "ExternalProgram(+options)+  RodDataFile");
    DefaultCtrl (hPanel, RUN_EXTPRG_TEXTMSG);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_WIDTH, 197);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RUN_EXTPRG_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RUN_EXTPRG_DECORATION_2
    RUN_EXTPRG_DECORATION_2 = NewCtrl (hPanel, CTRL_FLAT_BOX, "", 55, 62);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_CONSTANT_NAME, "DECORATION_2");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_FRAME_COLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_WIDTH, 214);

    // Build control: RUN_EXTPRG_TEXTMSG_2
    RUN_EXTPRG_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 249, 197);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_DFLT_VALUE, "sec");
    DefaultCtrl (hPanel, RUN_EXTPRG_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_WIDTH, 20);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RUN_EXTPRG_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_2, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION_2, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TEXTMSG_4, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DECORATION, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_ZPLANE_POSITION, 9);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RUN_EXTPRG_TIMEOUT, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_ROD_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_EXT_PRG, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_DATA_FILE_OPT, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, RUN_EXTPRG_WAIT_FOR_TERM, ATTR_CTRL_TAB_POSITION, 4);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(51*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(77*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_rwFpgaFlash ():  Build UI object RW_FPGAF; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_rwFpgaFlash (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RWFpgaFlash", 21, 47, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RW_FPGAF");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RW_FPGAF_OUT_FILE
    RW_FPGAF_OUT_FILE = NewCtrl (hPanel, CTRL_STRING, "OUTPUT FILE", 312, 9);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_CONSTANT_NAME, "OUT_FILE");
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWFpgaFlash);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_WIDTH, 73);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_FPGAF_OUT_FILE);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_TOP, 298);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_LEFT, 12);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_FPGAF_OUT_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_INP_FILE
    RW_FPGAF_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "INPUT FILE", 199, 9);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWFpgaFlash);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_FPGAF_INP_FILE);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_TOP, 185);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_FPGAF_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_SIZE
    RW_FPGAF_SIZE = NewCtrl (hPanel, CTRL_NUMERIC, "Size", 164, 134);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_CONSTANT_NAME, "SIZE");
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_WIDTH, 27);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_INCR_VALUE, 4);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_FPGAF_SIZE);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_TOP, 149);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_LEFT, 160);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_SIZE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_READ_NOT_WRITE
    RW_FPGAF_READ_NOT_WRITE = NewCtrl (hPanel, CTRL_VSWITCH, "", 60, 166);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_CONSTANT_NAME, "READ_NOT_WRITE");
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_CALLBACK_FUNCTION_POINTER, RWFpgaFlash);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_FPGAF_READ_NOT_WRITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_ON_TEXT, "READ");
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_OFF_TEXT, "WRITE");
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_TOP, 43);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_LEFT, 161);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_FPGAF_START_ADDRESS
    RW_FPGAF_START_ADDRESS = NewCtrl (hPanel, CTRL_NUMERIC, "StartAddress", 120, 144);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_CONSTANT_NAME, "START_ADDRESS");
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_CALLBACK_FUNCTION_POINTER, RWFpgaFlash);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_WIDTH, 75);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_PADDING, 6);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_FPGAF_START_ADDRESS);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_WIDTH, 55);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_TOP, 106);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_LEFT, 136);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_START_ADDRESS, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_DECORATION
    RW_FPGAF_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 30, 4);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_HEIGHT, 230);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_WIDTH, 342);

    // Build control: RW_FPGAF_TEXTMSG_4
    RW_FPGAF_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_DECORATION_2
    RW_FPGAF_DECORATION_2 = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 273, 4);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_CONSTANT_NAME, "DECORATION_2");
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_HEIGHT, 80);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_WIDTH, 342);

    // Build control: RW_FPGAF_TEXTMSG
    RW_FPGAF_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 268, 131);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_DFLT_VALUE, "OUTPUT DATA");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_WIDTH, 91);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_5
    RW_FPGAF_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 166, 207);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_DFLT_VALUE, "B");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_WIDTH, 10);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_2
    RW_FPGAF_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 167, 235);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_DFLT_VALUE, "FlashSize 0x80000 B");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_WIDTH, 102);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_3
    RW_FPGAF_TEXTMSG_3 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 43, 21);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_CONSTANT_NAME, "TEXTMSG_3");
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_VISIBLE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_COLOR, VAL_RED);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_DFLT_VALUE, "Address limits are set  during panel load (loadExitHost.c)!!");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_3);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_WIDTH, 271);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_3, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_6
    RW_FPGAF_TEXTMSG_6 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 108, 235);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_CONSTANT_NAME, "TEXTMSG_6");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_DFLT_VALUE, "Flash0: 0xE00000");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_6);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_WIDTH, 88);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_6, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_7
    RW_FPGAF_TEXTMSG_7 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 121, 235);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_CONSTANT_NAME, "TEXTMSG_7");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_DFLT_VALUE, "Flash1: 0xE80000");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_7);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_WIDTH, 88);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_7, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_8
    RW_FPGAF_TEXTMSG_8 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 133, 235);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_CONSTANT_NAME, "TEXTMSG_8");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_DFLT_VALUE, "Flash2: 0xF00000");
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_8);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_WIDTH, 87);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_8, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_FPGAF_TEXTMSG_12
    RW_FPGAF_TEXTMSG_12 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 220, 21);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_CONSTANT_NAME, "TEXTMSG_12");
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    dataPtr = 
        "Size is constant!!\nIf the file size has been changed, you have to edit this com"
        "mand!";
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_DFLT_VALUE, dataPtr);
    DefaultCtrl (hPanel, RW_FPGAF_TEXTMSG_12);
//  SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_HEIGHT, 28);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_WIDTH, 309);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_FPGAF_TEXTMSG_12, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_12, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_8, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_7, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_6, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_3, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_2, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_5, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION_2, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, RW_FPGAF_TEXTMSG_4, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, RW_FPGAF_DECORATION, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_ZPLANE_POSITION, 15);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RW_FPGAF_OUT_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_FPGAF_INP_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_FPGAF_SIZE, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_FPGAF_READ_NOT_WRITE, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_FPGAF_START_ADDRESS, ATTR_CTRL_TAB_POSITION, 4);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(21*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(47*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_rwMdsp ():  Build UI object RW_MASTERP; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_rwMdsp (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RWMaster", 11, 37, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RW_MASTERP");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RW_MASTERP_OUT_FILE
    RW_MASTERP_OUT_FILE = NewCtrl (hPanel, CTRL_STRING, "OUTPUT FILE", 400, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_CONSTANT_NAME, "OUT_FILE");
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_WIDTH, 73);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_MASTERP_OUT_FILE);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_TOP, 386);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_LEFT, 12);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_MASTERP_OUT_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_INP_FILE
    RW_MASTERP_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "INPUT FILE", 227, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_MASTERP_INP_FILE);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_TOP, 213);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_MASTERP_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_DATA_MASK
    RW_MASTERP_DATA_MASK = NewCtrl (hPanel, CTRL_NUMERIC, "DataMask", 455, 229);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_CONSTANT_NAME, "DATA_MASK");
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_WIDTH, 60);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_DFLT_VALUE, -1);
    DefaultCtrl (hPanel, RW_MASTERP_DATA_MASK);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_TOP, 457);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_LEFT, 168);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_DATA_MASK, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_COMP_DATA
    RW_MASTERP_COMP_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "CompData", 435, 229);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_CONSTANT_NAME, "COMP_DATA");
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_COMP_DATA);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_TOP, 438);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_LEFT, 167);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_COMP_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_INPUT_DATA
    RW_MASTERP_INPUT_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "InputData", 204, 138);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_CONSTANT_NAME, "INPUT_DATA");
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_WIDTH, 59);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_INPUT_DATA);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_TOP, 189);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_LEFT, 145);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_INPUT_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_START_ADDR
    RW_MASTERP_START_ADDR = NewCtrl (hPanel, CTRL_NUMERIC, "StartAddr", 116, 138);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_CONSTANT_NAME, "START_ADDR");
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_CALLBACK_FUNCTION_POINTER, wordBoundary);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_WIDTH, 56);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_START_ADDR);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_TOP, 101);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_LEFT, 146);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_START_ADDR, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_DELAY
    RW_MASTERP_DELAY = NewCtrl (hPanel, CTRL_NUMERIC, "Delay(ms)", 132, 250);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_CONSTANT_NAME, "DELAY");
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_WIDTH, 58);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_MAX_VALUE, 999);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_INCR_VALUE, 10);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_PADDING, 3);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_DELAY);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_WIDTH, 55);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_TOP, 117);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_LEFT, 250);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_DELAY, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_LENGTH
    RW_MASTERP_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "Length", 156, 138);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_CONSTANT_NAME, "LENGTH");
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_WIDTH, 42);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_INCR_VALUE, 4);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_DFLT_VALUE, 4);
    DefaultCtrl (hPanel, RW_MASTERP_LENGTH);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_TOP, 141);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_LEFT, 154);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_COMP_OPTION
    RW_MASTERP_COMP_OPTION = NewCtrl (hPanel, CTRL_CHECK_BOX, "COMPARE WITH", 436, 53);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_CONSTANT_NAME, "COMP_OPTION");
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_WIDTH, 88);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_COMP_OPTION);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_TOP, 437);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_LEFT, 69);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_OUTPUT_OPT
    RW_MASTERP_OUTPUT_OPT = NewCtrl (hPanel, CTRL_CHECK_BOX, "SAVE OUTPUT DATA TO FILE?", 343, 103);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_CONSTANT_NAME, "OUTPUT_OPT");
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_WIDTH, 160);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_MASTERP_OUTPUT_OPT);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_TOP, 344);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_LEFT, 120);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_INP_OPTION
    RW_MASTERP_INP_OPTION = NewCtrl (hPanel, CTRL_VSWITCH, "InputData", 190, 252);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_CONSTANT_NAME, "INP_OPTION");
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_DATA_TYPE, VAL_INTEGER);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_WIDTH, 50);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_MASTERP_INP_OPTION);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_ON_TEXT, "INPUT DATA");
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_OFF_TEXT, "FROM FILE");
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_TOP, 173);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_LEFT, 247);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_READ_NOT_WRITE
    RW_MASTERP_READ_NOT_WRITE = NewCtrl (hPanel, CTRL_VSWITCH, "", 57, 166);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_CONSTANT_NAME, "READ_NOT_WRITE");
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_MASTERP_READ_NOT_WRITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_ON_TEXT, "READ");
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_OFF_TEXT, "WRITE");
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_TOP, 40);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_LEFT, 161);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_BLOCK_ACCESS
    RW_MASTERP_BLOCK_ACCESS = NewCtrl (hPanel, CTRL_VSWITCH, "BlockAccess", 85, 252);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_CONSTANT_NAME, "BLOCK_ACCESS");
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_CALLBACK_FUNCTION_POINTER, RWMaster);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_WIDTH, 76);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_MASTERP_BLOCK_ACCESS);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_ON_TEXT, "Yes");
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_OFF_TEXT, "No (BOC)");
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_TOP, 68);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_LEFT, 250);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_WRITE_OR_APPEND
    RW_MASTERP_WRITE_OR_APPEND = NewCtrl (hPanel, CTRL_VSWITCH, "", 367, 104);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_CONSTANT_NAME, "WRITE_OR_APPEND");
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_BGCOLOR, 0XD5C9C4);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_TEXT_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_MASTERP_WRITE_OR_APPEND);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_ON_TEXT, "Append");
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_OFF_TEXT, "Write (deletes previous data!!)");
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_TOP, 346);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_LEFT, 84);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_MASTERP_DECORATION
    RW_MASTERP_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_HEIGHT, 260);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_WIDTH, 342);

    // Build control: RW_MASTERP_TEXTMSG_4
    RW_MASTERP_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RW_MASTERP_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_DECORATION_2
    RW_MASTERP_DECORATION_2 = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 301, 4);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_CONSTANT_NAME, "DECORATION_2");
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_HEIGHT, 190);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_WIDTH, 342);

    // Build control: RW_MASTERP_TEXTMSG
    RW_MASTERP_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 296, 131);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_DFLT_VALUE, "OUTPUT DATA");
    DefaultCtrl (hPanel, RW_MASTERP_TEXTMSG);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_WIDTH, 91);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_TEXTMSG_5
    RW_MASTERP_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 158, 211);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_DFLT_VALUE, "B");
    DefaultCtrl (hPanel, RW_MASTERP_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_WIDTH, 10);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_MASTERP_TEXTMSG_12
    RW_MASTERP_TEXTMSG_12 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 247, 22);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_CONSTANT_NAME, "TEXTMSG_12");
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    dataPtr = 
        "Length is constant!!\nIf the file size has been changed, you have to edit this c"
        "ommand!";
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_DFLT_VALUE, dataPtr);
    DefaultCtrl (hPanel, RW_MASTERP_TEXTMSG_12);
//  SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_HEIGHT, 28);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_WIDTH, 309);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_MASTERP_TEXTMSG_12, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_12, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_5, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION_2, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_ZPLANE_POSITION, 16);
    SetCtrlAttribute (hPanel, RW_MASTERP_TEXTMSG_4, ATTR_ZPLANE_POSITION, 17);
    SetCtrlAttribute (hPanel, RW_MASTERP_DECORATION, ATTR_ZPLANE_POSITION, 18);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_ZPLANE_POSITION, 19);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RW_MASTERP_OUT_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_MASTERP_DATA_MASK, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_DATA, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_MASTERP_INPUT_DATA, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_MASTERP_START_ADDR, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_MASTERP_DELAY, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_MASTERP_LENGTH, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_MASTERP_COMP_OPTION, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_MASTERP_OUTPUT_OPT, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_MASTERP_INP_OPTION, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_MASTERP_READ_NOT_WRITE, ATTR_CTRL_TAB_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_MASTERP_BLOCK_ACCESS, ATTR_CTRL_TAB_POSITION, 12);
    SetCtrlAttribute (hPanel, RW_MASTERP_WRITE_OR_APPEND, ATTR_CTRL_TAB_POSITION, 13);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(11*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(37*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_rwSdsp ():  Build UI object RW_SLAVEP; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_rwSdsp (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RWSlave", 21, 47, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RW_SLAVEP");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RW_SLAVEP_OUT_FILE
    RW_SLAVEP_OUT_FILE = NewCtrl (hPanel, CTRL_STRING, "OUTPUT FILE", 400, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_CONSTANT_NAME, "OUT_FILE");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_WIDTH, 73);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_SLAVEP_OUT_FILE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_TOP, 386);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_LEFT, 12);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_SLAVEP_OUT_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_INP_FILE
    RW_SLAVEP_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "INPUT FILE", 231, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_SLAVEP_INP_FILE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_TOP, 217);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_SLAVEP_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_COMP_DATA
    RW_SLAVEP_COMP_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "CompData", 441, 208);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_CONSTANT_NAME, "COMP_DATA");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_COMP_DATA);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_TOP, 426);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_LEFT, 215);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_COMP_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_INPUT_DATA
    RW_SLAVEP_INPUT_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "InputData", 208, 138);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_CONSTANT_NAME, "INPUT_DATA");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_WIDTH, 59);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_INPUT_DATA);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_TOP, 193);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_LEFT, 145);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_INPUT_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_START_ADDR
    RW_SLAVEP_START_ADDR = NewCtrl (hPanel, CTRL_NUMERIC, "StartAddr", 126, 138);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_CONSTANT_NAME, "START_ADDR");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_CALLBACK_FUNCTION_POINTER, wordBoundary);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_WIDTH, 56);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_START_ADDR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_TOP, 111);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_LEFT, 146);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_START_ADDR, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_LENGTH
    RW_SLAVEP_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "Length", 166, 138);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_CONSTANT_NAME, "LENGTH");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_WIDTH, 42);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_INCR_VALUE, 4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_DFLT_VALUE, 4);
    DefaultCtrl (hPanel, RW_SLAVEP_LENGTH);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_TOP, 151);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_LEFT, 154);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_COMP_OPTION
    RW_SLAVEP_COMP_OPTION = NewCtrl (hPanel, CTRL_CHECK_BOX, "COMPARE WITH", 443, 103);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_CONSTANT_NAME, "COMP_OPTION");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_WIDTH, 88);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_COMP_OPTION);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_TOP, 444);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_LEFT, 119);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_SLAVEP_OUTPUT_OPT
    RW_SLAVEP_OUTPUT_OPT = NewCtrl (hPanel, CTRL_CHECK_BOX, "SAVE OUTPUT DATA TO FILE?", 343, 103);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_CONSTANT_NAME, "OUTPUT_OPT");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_WIDTH, 160);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_OUTPUT_OPT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_TOP, 344);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_LEFT, 120);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_SLAVEP_INP_OPTION
    RW_SLAVEP_INP_OPTION = NewCtrl (hPanel, CTRL_VSWITCH, "InputData", 194, 252);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_CONSTANT_NAME, "INP_OPTION");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_DATA_TYPE, VAL_INTEGER);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_WIDTH, 50);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_SLAVEP_INP_OPTION);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_ON_TEXT, "INPUT DATA");
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_OFF_TEXT, "FROM FILE");
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_TOP, 177);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_LEFT, 247);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_SLAVEP_READ_NOT_WRITE
    RW_SLAVEP_READ_NOT_WRITE = NewCtrl (hPanel, CTRL_VSWITCH, "", 69, 166);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_CONSTANT_NAME, "READ_NOT_WRITE");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_CALLBACK_FUNCTION_POINTER, RWSlave);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_SLAVEP_READ_NOT_WRITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_ON_TEXT, "READ");
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_OFF_TEXT, "WRITE");
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_TOP, 52);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_LEFT, 161);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_SLAVEP_WRITE_OR_APPEND
    RW_SLAVEP_WRITE_OR_APPEND = NewCtrl (hPanel, CTRL_VSWITCH, "", 367, 104);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_CONSTANT_NAME, "WRITE_OR_APPEND");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_BGCOLOR, 0XD5C9C4);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_TEXT_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_WRITE_OR_APPEND);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_ON_TEXT, "Append");
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_OFF_TEXT, "Write (deletes previous data!!)");
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_TOP, 346);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_LEFT, 84);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_SLAVEP_SLAVE_NUMBER
    RW_SLAVEP_SLAVE_NUMBER = NewCtrl (hPanel, CTRL_RECESSED_MENU_RING, "Slave#", 68, 96);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_CONSTANT_NAME, "SLAVE_NUMBER");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_DATA_TYPE, VAL_UNSIGNED_CHAR);
    InsertListItem (hPanel, RW_SLAVEP_SLAVE_NUMBER, 0, "0", 0);
    InsertListItem (hPanel, RW_SLAVEP_SLAVE_NUMBER, 1, "1", 1);
    InsertListItem (hPanel, RW_SLAVEP_SLAVE_NUMBER, 2, "2", 2);
    InsertListItem (hPanel, RW_SLAVEP_SLAVE_NUMBER, 3, "3", 3);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_CTRL_MODE, VAL_NORMAL);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_WIDTH, 57);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_HEIGHT, 22);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_FONT, "Arial");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_POINT_SIZE, 22);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_BGCOLOR, VAL_WHITE);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_SLAVEP_SLAVE_NUMBER);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_MENU_ARROW_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_WIDTH, 50);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_TOP, 75);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_LEFT, 43);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: RW_SLAVEP_DECORATION
    RW_SLAVEP_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_HEIGHT, 260);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_WIDTH, 342);

    // Build control: RW_SLAVEP_TEXTMSG_4
    RW_SLAVEP_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RW_SLAVEP_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_TEXTMSG
    RW_SLAVEP_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 296, 131);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_DFLT_VALUE, "OUTPUT DATA");
    DefaultCtrl (hPanel, RW_SLAVEP_TEXTMSG);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_WIDTH, 91);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_TEXTMSG_5
    RW_SLAVEP_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 168, 211);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_DFLT_VALUE, "B");
    DefaultCtrl (hPanel, RW_SLAVEP_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_WIDTH, 10);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_SLAVEP_DECORATION_3
    RW_SLAVEP_DECORATION_3 = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 301, 4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_CONSTANT_NAME, "DECORATION_3");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_HEIGHT, 170);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_WIDTH, 342);

    // Build control: RW_SLAVEP_TEXTMSG_12
    RW_SLAVEP_TEXTMSG_12 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 252, 21);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_CONSTANT_NAME, "TEXTMSG_12");
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    dataPtr = 
        "Length is constant!!\nIf the file size has been changed, you have to edit this c"
        "ommand!";
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_DFLT_VALUE, dataPtr);
    DefaultCtrl (hPanel, RW_SLAVEP_TEXTMSG_12);
//  SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_HEIGHT, 28);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_WIDTH, 309);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_SLAVEP_TEXTMSG_12, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_12, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION_3, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_5, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, RW_SLAVEP_TEXTMSG_4, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, RW_SLAVEP_DECORATION, ATTR_ZPLANE_POSITION, 16);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_ZPLANE_POSITION, 17);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUT_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_DATA, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INPUT_DATA, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_SLAVEP_START_ADDR, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_SLAVEP_LENGTH, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_SLAVEP_COMP_OPTION, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_SLAVEP_OUTPUT_OPT, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_SLAVEP_INP_OPTION, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_SLAVEP_READ_NOT_WRITE, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_SLAVEP_WRITE_OR_APPEND, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_SLAVEP_SLAVE_NUMBER, ATTR_CTRL_TAB_POSITION, 11);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(21*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(47*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_rwVme ():  Build UI object RW_VMEP; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_rwVme (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "RWVme", 31, 57, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "RW_VMEP");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: RW_VMEP_OUT_FILE
    RW_VMEP_OUT_FILE = NewCtrl (hPanel, CTRL_STRING, "OUTPUT FILE", 440, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_CONSTANT_NAME, "OUT_FILE");
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_WIDTH, 73);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_VMEP_OUT_FILE);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_TOP, 426);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_LEFT, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_VMEP_OUT_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_INP_FILE
    RW_VMEP_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "INPUT FILE", 290, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, RW_VMEP_INP_FILE);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_TOP, 276);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, RW_VMEP_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_COMP_DATA
    RW_VMEP_COMP_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "CompData", 481, 208);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_CONSTANT_NAME, "COMP_DATA");
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_VMEP_COMP_DATA);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_TOP, 466);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_LEFT, 215);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_COMP_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_INPUT_DATA
    RW_VMEP_INPUT_DATA = NewCtrl (hPanel, CTRL_NUMERIC, "InputData", 267, 138);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_CONSTANT_NAME, "INPUT_DATA");
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_WIDTH, 59);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_VMEP_INPUT_DATA);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_TOP, 252);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_LEFT, 145);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_INPUT_DATA, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_START_ADDR
    RW_VMEP_START_ADDR = NewCtrl (hPanel, CTRL_NUMERIC, "SlotRelStartAddr", 185, 138);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_CONSTANT_NAME, "START_ADDR");
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_WIDTH, 97);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_MAX_VALUE, 16777215);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_PADDING, 6);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_VMEP_START_ADDR);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_TOP, 170);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_LEFT, 126);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_START_ADDR, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_LENGTH
    RW_VMEP_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "Length", 225, 138);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_CONSTANT_NAME, "LENGTH");
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_WIDTH, 42);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_INCR_VALUE, 4);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_DFLT_VALUE, 4);
    DefaultCtrl (hPanel, RW_VMEP_LENGTH);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_TOP, 210);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_LEFT, 154);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_COMP_OPTION
    RW_VMEP_COMP_OPTION = NewCtrl (hPanel, CTRL_CHECK_BOX, "COMPARE WITH", 483, 103);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_CONSTANT_NAME, "COMP_OPTION");
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_WIDTH, 88);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_VMEP_COMP_OPTION);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_TOP, 484);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_LEFT, 119);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_OUTPUT_OPT
    RW_VMEP_OUTPUT_OPT = NewCtrl (hPanel, CTRL_CHECK_BOX, "SAVE OUTPUT DATA TO FILE?", 383, 103);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_CONSTANT_NAME, "OUTPUT_OPT");
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_WIDTH, 160);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, RW_VMEP_OUTPUT_OPT);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_ON_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_OFF_COLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_HEIGHT, 16);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_WIDTH, 16);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_TOP, 384);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_LEFT, 120);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_INP_OPTION
    RW_VMEP_INP_OPTION = NewCtrl (hPanel, CTRL_VSWITCH, "InputData", 253, 252);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_CONSTANT_NAME, "INP_OPTION");
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_DATA_TYPE, VAL_INTEGER);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_DIMMED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_WIDTH, 50);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_VMEP_INP_OPTION);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_ON_TEXT, "INPUT DATA");
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_OFF_TEXT, "FROM FILE");
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_TOP, 236);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_LEFT, 247);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_NO_BLOCK_ACCESS
    RW_VMEP_NO_BLOCK_ACCESS = NewCtrl (hPanel, CTRL_VSWITCH, "BlockAccess", 115, 186);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_CONSTANT_NAME, "NO_BLOCK_ACCESS");
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_DATA_TYPE, VAL_UNSIGNED_CHAR);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_WIDTH, 76);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_TEXT_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_VMEP_NO_BLOCK_ACCESS);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_ON_TEXT, "No (TIM)");
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_OFF_TEXT, "Yes");
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_WIDTH, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_TOP, 97);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_LEFT, 172);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_READ_NOT_WRITE
    RW_VMEP_READ_NOT_WRITE = NewCtrl (hPanel, CTRL_VSWITCH, "", 58, 166);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_CONSTANT_NAME, "READ_NOT_WRITE");
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, RW_VMEP_READ_NOT_WRITE);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_ON_TEXT, "READ");
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_OFF_TEXT, "WRITE");
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_TOP, 41);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_LEFT, 161);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_WRITE_OR_APPEND
    RW_VMEP_WRITE_OR_APPEND = NewCtrl (hPanel, CTRL_VSWITCH, "", 407, 104);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_CONSTANT_NAME, "WRITE_OR_APPEND");
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_BGCOLOR, 0XD5C9C4);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_TEXT_BOLD, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_VMEP_WRITE_OR_APPEND);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_ON_TEXT, "Append");
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_OFF_TEXT, "Write (deletes previous data!!)");
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_HEIGHT, 30);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_WIDTH, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_TOP, 386);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_LEFT, 84);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_VME_ACCESS
    RW_VMEP_VME_ACCESS = NewCtrl (hPanel, CTRL_RING_POINTER_VSLIDE, "ElementSize", 116, 145);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_CONSTANT_NAME, "VME_ACCESS");
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_CALLBACK_FUNCTION_POINTER, RWVme);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_DATA_TYPE, VAL_UNSIGNED_SHORT_INTEGER);
    InsertListItem (hPanel, RW_VMEP_VME_ACCESS, 0, "8 bit", 1);
    InsertListItem (hPanel, RW_VMEP_VME_ACCESS, 1, "16 bit", 2);
    InsertListItem (hPanel, RW_VMEP_VME_ACCESS, 2, "32 bit", 4);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_CTRL_MODE, VAL_NORMAL);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_WIDTH, 72);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TEXT_BOLD, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, RW_VMEP_VME_ACCESS);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_NEEDLE_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_FILL_HOUSING_COLOR, VAL_GRAY);
//  SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_TICK_STYLE, VAL_NO_MINOR_TICKS);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_FILL_COLOR, VAL_RED);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_FILL_OPTION, VAL_NO_FILL);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_HEIGHT, 26);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_WIDTH, 7);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_TOP, 97);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_LEFT, 93);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_SLIDER_LEFT, 137);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_SLIDER_WIDTH, 17);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_SLIDER_HEIGHT, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: RW_VMEP_DECORATION
    RW_VMEP_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_HEIGHT, 320);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_WIDTH, 342);

    // Build control: RW_VMEP_TEXTMSG_4
    RW_VMEP_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_4);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_TEXTMSG
    RW_VMEP_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 357, 131);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_DFLT_VALUE, "OUTPUT DATA");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_WIDTH, 91);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_TEXTMSG_5
    RW_VMEP_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 227, 211);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_DFLT_VALUE, "B");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_WIDTH, 10);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_DECORATION_3
    RW_VMEP_DECORATION_3 = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 362, 4);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_CONSTANT_NAME, "DECORATION_3");
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_HEIGHT, 150);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_WIDTH, 342);

    // Build control: RW_VMEP_TEXTMSG_2
    RW_VMEP_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 175, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_COLOR, 0X303030);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_DFLT_VALUE, "FpgaCnfg    0xC00000");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_WIDTH, 109);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_TEXTMSG_6
    RW_VMEP_TEXTMSG_6 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 188, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_CONSTANT_NAME, "TEXTMSG_6");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_COLOR, 0X303030);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_DFLT_VALUE, "FpgaRst      0xC00004");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_6);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_WIDTH, 109);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_6, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_TEXTMSG_7
    RW_VMEP_TEXTMSG_7 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 200, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_CONSTANT_NAME, "TEXTMSG_7");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_COLOR, 0X303030);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_DFLT_VALUE, "VmeDspRst 0xC00008");
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_7);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_WIDTH, 110);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_7, "", VAL_DS_READ, NULL, NULL);

    // Build control: RW_VMEP_TEXTMSG_12
    RW_VMEP_TEXTMSG_12 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 312, 21);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_CONSTANT_NAME, "TEXTMSG_12");
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    dataPtr = 
        "Length is constant!!\nIf the file size has been changed, you have to edit this c"
        "ommand!";
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_DFLT_VALUE, dataPtr);
    DefaultCtrl (hPanel, RW_VMEP_TEXTMSG_12);
//  SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_HEIGHT, 28);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_WIDTH, 309);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, RW_VMEP_TEXTMSG_12, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_12, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_7, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_6, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_2, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION_3, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_5, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_ZPLANE_POSITION, 16);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_ZPLANE_POSITION, 17);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_ZPLANE_POSITION, 18);
    SetCtrlAttribute (hPanel, RW_VMEP_TEXTMSG_4, ATTR_ZPLANE_POSITION, 19);
    SetCtrlAttribute (hPanel, RW_VMEP_DECORATION, ATTR_ZPLANE_POSITION, 20);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_ZPLANE_POSITION, 21);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, RW_VMEP_OUT_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_FILE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_DATA, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, RW_VMEP_INPUT_DATA, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, RW_VMEP_START_ADDR, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, RW_VMEP_LENGTH, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, RW_VMEP_COMP_OPTION, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, RW_VMEP_OUTPUT_OPT, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, RW_VMEP_INP_OPTION, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, RW_VMEP_NO_BLOCK_ACCESS, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, RW_VMEP_READ_NOT_WRITE, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, RW_VMEP_WRITE_OR_APPEND, ATTR_CTRL_TAB_POSITION, 11);
    SetCtrlAttribute (hPanel, RW_VMEP_VME_ACCESS, ATTR_CTRL_TAB_POSITION, 12);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(31*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(57*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_sendPrimList ():  Build UI object SEND_PLIST; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_sendPrimList (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "SendPrimList", 11, 37, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "SEND_PLIST");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: SEND_PLIST_INP_FILE
    SEND_PLIST_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "PrimList", 154, 9);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, sendPrimList);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_WIDTH, 47);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, SEND_PLIST_INP_FILE);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_TOP, 140);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_LEFT, 145);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, SEND_PLIST_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: SEND_PLIST_REPEAT_BUILD_EXEC
    SEND_PLIST_REPEAT_BUILD_EXEC = NewCtrl (hPanel, CTRL_VSWITCH, "Repeat", 86, 152);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_CONSTANT_NAME, "REPEAT_BUILD_EXEC");
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_DATA_TYPE, VAL_INTEGER);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_POINT_SIZE, 9);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_WIDTH, 44);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_FONT, VAL_APP_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_POINT_SIZE, 10);
//  SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_DFLT_INDEX, 1);
    DefaultCtrl (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_ON_VALUE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_OFF_VALUE, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_BINARY_SWITCH_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_ON_TEXT, "EXECUTION");
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_OFF_TEXT, "BUILDING");
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_HEIGHT, 28);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_WIDTH, 13);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_TOP, 71);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_LEFT, 147);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_LABEL_SIZE_TO_TEXT, 1);

    // Build control: SEND_PLIST_EDIT_PRIM_LIST
    SEND_PLIST_EDIT_PRIM_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "Edit PrimList", 204, 140);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_CONSTANT_NAME, "EDIT_PRIM_LIST");
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_CALLBACK_FUNCTION_POINTER, sendPrimList);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SEND_PLIST_EDIT_PRIM_LIST);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_WIDTH, 68);

    // Build control: SEND_PLIST_DECORATION
    SEND_PLIST_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_HEIGHT, 160);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_WIDTH, 342);

    // Build control: SEND_PLIST_TEXTMSG_4
    SEND_PLIST_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, SEND_PLIST_TEXTMSG_4);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, SEND_PLIST_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_TEXTMSG_4, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, SEND_PLIST_DECORATION, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_ZPLANE_POSITION, 4);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, SEND_PLIST_INP_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, SEND_PLIST_REPEAT_BUILD_EXEC, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, SEND_PLIST_EDIT_PRIM_LIST, ATTR_CTRL_TAB_POSITION, 2);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(11*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(37*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_wMdspFlash ():  Build UI object W_MDSPF; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_wMdspFlash (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "WMDSPFlash", 31, 57, 525, 350);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "W_MDSPF");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XCACACA);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: W_MDSPF_INP_FILE
    W_MDSPF_INP_FILE = NewCtrl (hPanel, CTRL_STRING, "INPUT FILE", 147, 9);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_CONSTANT_NAME, "INP_FILE");
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_CALLBACK_FUNCTION_POINTER, WMDSPFlash);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_WIDTH, 61);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, W_MDSPF_INP_FILE);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_MAX_ENTRY_LENGTH, 99);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_WIDTH, 330);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_TOP, 133);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_LEFT, 11);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, W_MDSPF_INP_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_SIZE
    W_MDSPF_SIZE = NewCtrl (hPanel, CTRL_NUMERIC, "Size", 112, 134);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_CONSTANT_NAME, "SIZE");
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_WIDTH, 27);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_MAX_VALUE, 524288);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_INCR_VALUE, 4);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_CHECK_RANGE, VAL_COERCE);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, W_MDSPF_SIZE);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_WIDTH, 72);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_TOP, 97);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_LEFT, 159);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_SIZE, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_START_ADDRESS
    W_MDSPF_START_ADDRESS = NewCtrl (hPanel, CTRL_NUMERIC, "StartAddress", 70, 134);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_CONSTANT_NAME, "START_ADDRESS");
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_WIDTH, 75);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_PADDING, 8);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, W_MDSPF_START_ADDRESS);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_TOP, 56);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_LEFT, 131);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_START_ADDRESS, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_DECORATION
    W_MDSPF_DECORATION = NewCtrl (hPanel, CTRL_RECESSED_FRAME, "", 28, 4);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_CONSTANT_NAME, "DECORATION");
//  SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_FRAME_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_HEIGHT, 160);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_WIDTH, 342);

    // Build control: W_MDSPF_TEXTMSG_4
    W_MDSPF_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 135);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_DFLT_VALUE, "INPUT DATA");
    DefaultCtrl (hPanel, W_MDSPF_TEXTMSG_4);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_RAISED, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_WIDTH, 78);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_TEXTMSG_5
    W_MDSPF_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 114, 207);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_DFLT_VALUE, "B");
    DefaultCtrl (hPanel, W_MDSPF_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_WIDTH, 10);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_TEXTMSG_2
    W_MDSPF_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 115, 235);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_DFLT_VALUE, "FlashSize 0x80000 B");
    DefaultCtrl (hPanel, W_MDSPF_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_WIDTH, 102);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_TEXTMSG_3
    W_MDSPF_TEXTMSG_3 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 38, 40);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_CONSTANT_NAME, "TEXTMSG_3");
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_VISIBLE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_COLOR, VAL_RED);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_DFLT_VALUE, "Address limits are set  during panel load (loadExitHost.c)!!");
    DefaultCtrl (hPanel, W_MDSPF_TEXTMSG_3);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_WIDTH, 271);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_TEXTMSG_3, "", VAL_DS_READ, NULL, NULL);

    // Build control: W_MDSPF_TEXTMSG_6
    W_MDSPF_TEXTMSG_6 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 166, 59);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_CONSTANT_NAME, "TEXTMSG_6");
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_COLOR, 0X000066);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_BGCOLOR, 0XCACACA);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_DFLT_VALUE, "Always writes the whole file! (Size is not constant)");
    DefaultCtrl (hPanel, W_MDSPF_TEXTMSG_6);
//  SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_WIDTH, 235);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, W_MDSPF_TEXTMSG_6, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_6, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_3, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_2, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_5, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, W_MDSPF_TEXTMSG_4, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, W_MDSPF_DECORATION, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_ZPLANE_POSITION, 8);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, W_MDSPF_INP_FILE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, W_MDSPF_SIZE, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, W_MDSPF_START_ADDRESS, ATTR_CTRL_TAB_POSITION, 2);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(525*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(350*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(31*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(57*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}
