
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------
//
//  "uiPrimListEditor.c"
//
//  Implementation file for UI build functions
//
//  Date: 11-07-2004
//
//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//  Include header files
//----------------------------------------------------------------------------------------------------

#include "uiPrimListEditor.h"
#include <Utility.h> // definition of NULL
#include "toolbox.h" // definition of errChk macro

extern int CVIFUNC_C GetUserBitmapFromText (char *imageText, int version, int *bitmapId); // needed if there are bitmaps
extern int CVIFUNC_C NewActiveXCtrlFromPersistence (int panel, const char *ctrlLabel,
                                                    int top, int left, char *defaultIID,
                                                    int binaryFormat, char *persistenceText,
                                                    HRESULT *activeXError); // needed if there are ActiveX controls

//----------------------------------------------------------------------------------------------------
//  Prototype UI build functions.
//----------------------------------------------------------------------------------------------------

int buildPanel_primListEditor (int hParentPanel);
int buildPanel_sdspListEditor (int hParentPanel);
int buildMenu_primListEditor (int hParentPanel);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: LIST_EDIT
// ------------------------------------------------------------------------------------------------

// Control: LIST_EDIT_PRIM_RING_SLAVE
int CVICALLBACK primRing (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_PRIM_RING_MASTER
int CVICALLBACK primRing (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_CLEAR_LCOMM
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_CLEAR_PCOMM
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_CLEAR_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_SAVE_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_LOAD_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_INSERT_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_SEND_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_REPLACE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_ADD_AFTER
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_ADD_BEFORE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_EDIT_PRIMITIVE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_DELETE_PRIMITIVE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: LIST_EDIT_LIST_TABLE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Panel: SLAVE_LST
// ------------------------------------------------------------------------------------------------

// Control: SLAVE_LST_CLEAR_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_SAVE_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_LOAD_LIST
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_REPLACE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_ADD_AFTER
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_ADD_BEFORE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_LIST_TABLE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_SLIST_DONE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_EDIT_PRIMITIVE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

// Control: SLAVE_LST_DELETE_PRIMITIVE
int CVICALLBACK listEditWindowButtons (int panelHandle, int control, int event, void *callbackData, int eventData1, int eventData2);

//-------------------------------------------------------------------------------------------------
// Prototype callbacks for Menubar: LIST_EDITM
// ------------------------------------------------------------------------------------------------

// Menu: GOTO_MAIN
void CVICALLBACK listEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: COMM_LIST_EDIT
void CVICALLBACK listEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: DATA_FILE_CREATOR
void CVICALLBACK listEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: REVISION_UPDATE
void CVICALLBACK listEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);

// Menu: CLOSE
void CVICALLBACK listEditMenu (int menubarHandle, int menuItemID, void *callbackPtr, int panelHandle);


//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  LIST_EDIT
// ------------------------------------------------------------------------------------------------

int LIST_EDIT_PRIM_RING_SLAVE                = 0;  // control identifier
int LIST_EDIT_PRIM_RING_MASTER               = 0;  // control identifier
int LIST_EDIT_CLEAR_LCOMM                    = 0;  // control identifier
int LIST_EDIT_CLEAR_PCOMM                    = 0;  // control identifier
int LIST_EDIT_CLEAR_LIST                     = 0;  // control identifier
int LIST_EDIT_SAVE_LIST                      = 0;  // control identifier
int LIST_EDIT_LOAD_LIST                      = 0;  // control identifier
int LIST_EDIT_INSERT_LIST                    = 0;  // control identifier
int LIST_EDIT_SEND_LIST                      = 0;  // control identifier
int LIST_EDIT_REPLACE                        = 0;  // control identifier
int LIST_EDIT_ADD_AFTER                      = 0;  // control identifier
int LIST_EDIT_ADD_BEFORE                     = 0;  // control identifier
int LIST_EDIT_REPLIST_LENGTH                 = 0;  // control identifier
int LIST_EDIT_REPPRIM_COUNT                  = 0;  // control identifier
int LIST_EDIT_ALL_PRIM_COUNT                 = 0;  // control identifier
int LIST_EDIT_PRIM_COUNT                     = 0;  // control identifier
int LIST_EDIT_PRIMLIST_LENGTH                = 0;  // control identifier
int LIST_EDIT_EDIT_PRIMITIVE                 = 0;  // control identifier
int LIST_EDIT_DELETE_PRIMITIVE               = 0;  // control identifier
int LIST_EDIT_LIST_TABLE                     = 0;  // control identifier
int LIST_EDIT_DSP                            = 0;  // control identifier
int LIST_EDIT_SLOT_NUMBER                    = 0;  // control identifier
int LIST_EDIT_PRIM_COMMENT                   = 0;  // control identifier
int LIST_EDIT_LIST_COMMENT                   = 0;  // control identifier
int LIST_EDIT_LAST_FILE                      = 0;  // control identifier
int LIST_EDIT_PLIST_REP                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_2                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_3                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_4                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG                        = 0;  // control identifier
int LIST_EDIT_DECORATION_3                   = 0;  // control identifier
int LIST_EDIT_TEXTMSG_6                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_7                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_8                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_9                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_5                      = 0;  // control identifier
int LIST_EDIT_TEXTMSG_11                     = 0;  // control identifier
int LIST_EDIT_TEXTMSG_12                     = 0;  // control identifier
int LIST_EDIT_TEXTMSG_13                     = 0;  // control identifier
int LIST_EDIT_TEXTMSG_14                     = 0;  // control identifier
int LIST_EDIT_TEXTMSG_10                     = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Panel:  SLAVE_LST
// ------------------------------------------------------------------------------------------------

int SLAVE_LST_REPLIST_LENGTH                 = 0;  // control identifier
int SLAVE_LST_REPPRIM_COUNT                  = 0;  // control identifier
int SLAVE_LST_CLEAR_LIST                     = 0;  // control identifier
int SLAVE_LST_SAVE_LIST                      = 0;  // control identifier
int SLAVE_LST_LOAD_LIST                      = 0;  // control identifier
int SLAVE_LST_PRIM_COUNT                     = 0;  // control identifier
int SLAVE_LST_REPLACE                        = 0;  // control identifier
int SLAVE_LST_ADD_AFTER                      = 0;  // control identifier
int SLAVE_LST_ADD_BEFORE                     = 0;  // control identifier
int SLAVE_LST_PRIMLIST_LENGTH                = 0;  // control identifier
int SLAVE_LST_LIST_TABLE                     = 0;  // control identifier
int SLAVE_LST_SLIST_DONE                     = 0;  // control identifier
int SLAVE_LST_EDIT_PRIMITIVE                 = 0;  // control identifier
int SLAVE_LST_DELETE_PRIMITIVE               = 0;  // control identifier
int SLAVE_LST_LAST_FILE                      = 0;  // control identifier
int SLAVE_LST_TEXTMSG_2                      = 0;  // control identifier
int SLAVE_LST_TEXTMSG                        = 0;  // control identifier

//-------------------------------------------------------------------------------------------------
// Declare identifiers for Menubar:  LIST_EDITM
// ------------------------------------------------------------------------------------------------

int LIST_EDITM_GOTO_MAIN                     = 0;  // menu identifier
int LIST_EDITM_COMM_LIST_EDIT                = 0;  // menu identifier
int LIST_EDITM_DATA_FILE_CREATOR             = 0;  // menu identifier
int LIST_EDITM_REVISION_UPDATE               = 0;  // menu identifier
int LIST_EDITM_CLOSE                         = 0;  // menu identifier

//----------------------------------------------------------------------------------------------------
// buildPanel_primListEditor ():  Build UI object LIST_EDIT; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_primListEditor (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "PRIM LIST EDITOR", 29, 13, 708, 765);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "LIST_EDIT");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XE6D1B3);
//  SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_OFFWHITE);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_GRAY);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_RAISED_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 10);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
//  SetPanelAttribute (hPanel, ATTR_MOVABLE, 1);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
//  SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
    SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 1);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: LIST_EDIT_PRIM_RING_SLAVE
    LIST_EDIT_PRIM_RING_SLAVE = NewCtrl (hPanel, CTRL_RING, "", 73, 8);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_CONSTANT_NAME, "PRIM_RING_SLAVE");
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_VISIBLE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_CALLBACK_FUNCTION_POINTER, primRing);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_COLOR, 0X202020);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_RAISED, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_BGCOLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_POINT_SIZE, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_WIDTH, 353);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_TOP, 55);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_LEFT, 266);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: LIST_EDIT_PRIM_RING_MASTER
    LIST_EDIT_PRIM_RING_MASTER = NewCtrl (hPanel, CTRL_RING, "", 73, 8);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_CONSTANT_NAME, "PRIM_RING_MASTER");
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_CALLBACK_FUNCTION_POINTER, primRing);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_COLOR, 0X202020);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_RAISED, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_BGCOLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_POINT_SIZE, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_WIDTH, 353);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_TOP, 55);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_LEFT, 266);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: LIST_EDIT_CLEAR_LCOMM
    LIST_EDIT_CLEAR_LCOMM = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "Clear", 663, 424);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_CONSTANT_NAME, "CLEAR_LCOMM");
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_CLEAR_LCOMM);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_WIDTH, 38);

    // Build control: LIST_EDIT_CLEAR_PCOMM
    LIST_EDIT_CLEAR_PCOMM = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "Clear", 663, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_CONSTANT_NAME, "CLEAR_PCOMM");
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_CLEAR_PCOMM);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_HEIGHT, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_WIDTH, 38);

    // Build control: LIST_EDIT_CLEAR_LIST
    LIST_EDIT_CLEAR_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE LIST", 473, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_CONSTANT_NAME, "CLEAR_LIST");
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_CLEAR_LIST);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_SAVE_LIST
    LIST_EDIT_SAVE_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "SAVE LIST", 437, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_CONSTANT_NAME, "SAVE_LIST");
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_SAVE_LIST);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_LOAD_LIST
    LIST_EDIT_LOAD_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "LOAD LIST", 383, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_CONSTANT_NAME, "LOAD_LIST");
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_LOAD_LIST);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_INSERT_LIST
    LIST_EDIT_INSERT_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "INSERT LIST", 410, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_CONSTANT_NAME, "INSERT_LIST");
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_INSERT_LIST);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_SEND_LIST
    LIST_EDIT_SEND_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "SEND TO ROD", 115, 367);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_CONSTANT_NAME, "SEND_LIST");
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_SEND_LIST);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_AUTO_SIZING, VAL_NEVER_AUTO_SIZE);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_WIDTH, 92);

    // Build control: LIST_EDIT_REPLACE
    LIST_EDIT_REPLACE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "REPLACE>>", 199, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_CONSTANT_NAME, "REPLACE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_SHORTCUT_KEY, VAL_MENUKEY_MODIFIER | VAL_RIGHT_ARROW_VKEY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_REPLACE);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_ADD_AFTER
    LIST_EDIT_ADD_AFTER = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD AFTER>>", 226, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_CONSTANT_NAME, "ADD_AFTER");
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_SHORTCUT_KEY, VAL_MENUKEY_MODIFIER | VAL_DOWN_ARROW_VKEY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_ADD_AFTER);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_ADD_BEFORE
    LIST_EDIT_ADD_BEFORE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD BEFORE>", 172, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_CONSTANT_NAME, "ADD_BEFORE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_SHORTCUT_KEY, VAL_MENUKEY_MODIFIER | VAL_UP_ARROW_VKEY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_ADD_BEFORE);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_REPLIST_LENGTH
    LIST_EDIT_REPLIST_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "RepList", 76, 559);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_CONSTANT_NAME, "REPLIST_LENGTH");
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_WIDTH, 46);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_REPLIST_LENGTH);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_TOP, 78);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_LEFT, 510);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_REPLIST_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_REPPRIM_COUNT
    LIST_EDIT_REPPRIM_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "", 76, 634);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_CONSTANT_NAME, "REPPRIM_COUNT");
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_REPPRIM_COUNT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_TOP, 62);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_LEFT, 666);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_REPPRIM_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_ALL_PRIM_COUNT
    LIST_EDIT_ALL_PRIM_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "AllPrimCount", 56, 699);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_CONSTANT_NAME, "ALL_PRIM_COUNT");
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_WIDTH, 62);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_ALL_PRIM_COUNT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_TOP, 41);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_LEFT, 688);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_ALL_PRIM_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_PRIM_COUNT
    LIST_EDIT_PRIM_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "PrimCount", 56, 634);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_CONSTANT_NAME, "PRIM_COUNT");
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_WIDTH, 51);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_PRIM_COUNT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_TOP, 41);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_LEFT, 629);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_PRIM_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_PRIMLIST_LENGTH
    LIST_EDIT_PRIMLIST_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "PrimList", 56, 559);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_CONSTANT_NAME, "PRIMLIST_LENGTH");
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_WIDTH, 47);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_PRIMLIST_LENGTH);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_TOP, 59);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_LEFT, 510);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_PRIMLIST_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_EDIT_PRIMITIVE
    LIST_EDIT_EDIT_PRIMITIVE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "<<EDIT PRIM", 284, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_CONSTANT_NAME, "EDIT_PRIMITIVE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_SHORTCUT_KEY, VAL_MENUKEY_MODIFIER | VAL_LEFT_ARROW_VKEY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_EDIT_PRIMITIVE);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_DELETE_PRIMITIVE
    LIST_EDIT_DELETE_PRIMITIVE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE PRIM", 320, 364);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_CONSTANT_NAME, "DELETE_PRIMITIVE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_SHORTCUT_KEY, VAL_MENUKEY_MODIFIER | VAL_FWD_DELETE_VKEY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_DELETE_PRIMITIVE);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_WIDTH, 98);

    // Build control: LIST_EDIT_LIST_TABLE
    LIST_EDIT_LIST_TABLE = NewCtrl (hPanel, CTRL_TABLE, "", 117, 464);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CONSTANT_NAME, "LIST_TABLE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_SCROLL_BARS, VAL_VERT_SCROLL_BAR);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_SCROLL_BAR_SIZE, VAL_LARGE_SCROLL_BARS);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_TEXT, "");
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_MAX_VALUE, HUGE_VAL);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_MIN_VALUE, -HUGE_VAL);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_INCR_VALUE, 1.000000);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_PRECISION, 2);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_WRAP_MODE, VAL_LINE_WRAP);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ENTER_IS_NEWLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_NO_EDIT_TEXT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_UPPER_LEFT_CORNER_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TABLE_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_TABLE_MODE, VAL_COLUMN);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ROW_LABELS_WIDTH, 22);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ENABLE_ROW_SIZING, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_COLUMN_LABELS_HEIGHT, 25);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ENABLE_COLUMN_SIZING, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ENABLE_POPUP_MENU, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CELL_TYPE, VAL_CELL_STRING);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CELL_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CELL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_STR_CELL_DFLT_VALUE, "");
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CELL_JUSTIFY, VAL_TOP_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_VERTICAL_GRID_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_HEIGHT, 530);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_WIDTH, 291);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_TOP, 96);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_LEFT, 598);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    InsertTableColumns (hPanel, LIST_EDIT_LIST_TABLE, -1, 3, VAL_USE_MASTER_CELL_TYPE);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_TEXT, "ID           PRIMITIVE");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_MAX_VALUE, HUGE_VAL);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_MIN_VALUE, -HUGE_VAL);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_INCR_VALUE, 1.000000);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_PRECISION, 2);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_COLUMN_WIDTH, 170);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_CELL_TYPE, VAL_CELL_STRING);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_CELL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 1, ATTR_VERTICAL_GRID_VISIBLE, 1);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_TEXT, "PrimLn");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_MAX_VALUE, -1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_MIN_VALUE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_INCR_VALUE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_PRECISION, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_COLUMN_WIDTH, 39);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_CELL_TYPE, VAL_CELL_NUMERIC);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_NUM_CELL_DFLT_VALUE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_CELL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 2, ATTR_VERTICAL_GRID_VISIBLE, 1);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_TEXT, "RepLn");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_MAX_VALUE, -1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_MIN_VALUE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_INCR_VALUE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_PRECISION, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_COLUMN_WIDTH, 38);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_CELL_TYPE, VAL_CELL_NUMERIC);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_NUM_CELL_DFLT_VALUE, 0);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_CELL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, LIST_EDIT_LIST_TABLE, 3, ATTR_VERTICAL_GRID_VISIBLE, 1);
    DefaultCtrl (hPanel, LIST_EDIT_LIST_TABLE);
    SetActiveTableCell (hPanel, LIST_EDIT_LIST_TABLE, MakePoint (3, 0));
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_VSCROLL_OFFSET, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindTableCellRange (hPanel, LIST_EDIT_LIST_TABLE, MakeRect(0, 0, 0, 0), "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_DSP
    LIST_EDIT_DSP = NewCtrl (hPanel, CTRL_RECESSED_MENU_RING, "", 48, 144);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_CONSTANT_NAME, "DSP");
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_DATA_TYPE, VAL_UNSIGNED_CHAR);
    InsertListItem (hPanel, LIST_EDIT_DSP, 0, "MASTER", 0);
    InsertListItem (hPanel, LIST_EDIT_DSP, 1, "SLAVE", 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_COLOR, 0X000066);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_POINT_SIZE, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_BGCOLOR, VAL_WHITE);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, LIST_EDIT_DSP);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_MENU_ARROW_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_WIDTH, 92);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_TOP, 53);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_LEFT, 284);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: LIST_EDIT_SLOT_NUMBER
    LIST_EDIT_SLOT_NUMBER = NewCtrl (hPanel, CTRL_RECESSED_MENU_RING, "Slot#", 41, 378);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_CONSTANT_NAME, "SLOT_NUMBER");
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_DATA_TYPE, VAL_UNSIGNED_CHAR);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 0, "ALL", 0);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 1, "5", 5);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 2, "6", 6);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 3, "7", 7);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 4, "8", 8);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 5, "9", 9);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 6, "10", 10);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 7, "11", 11);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 8, "12", 12);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 9, "14", 14);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 10, "15", 15);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 11, "16", 16);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 12, "17", 17);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 13, "18", 18);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 14, "19", 19);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 15, "20", 20);
    InsertListItem (hPanel, LIST_EDIT_SLOT_NUMBER, 16, "21", 21);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_FRAME_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_DISABLE_CHECK_MARK, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_CTRL_MODE, VAL_NORMAL);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_POINT_SIZE, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_WIDTH, 38);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_HEIGHT, 18);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_COLOR, 0XFFFF99);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_FONT, "Arial");
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_POINT_SIZE, 22);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_BGCOLOR, 0X003366);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_DFLT_INDEX, 0);
    DefaultCtrl (hPanel, LIST_EDIT_SLOT_NUMBER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_MENU_ARROW_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_TOP, 25);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_LEFT, 390);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);

    // Build control: LIST_EDIT_PRIM_COMMENT
    LIST_EDIT_PRIM_COMMENT = NewCtrl (hPanel, CTRL_TEXT_BOX, "", 652, 8);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_CONSTANT_NAME, "PRIM_COMMENT");
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_SCROLL_BARS, VAL_BOTH_SCROLL_BARS);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_SCROLL_BAR_SIZE, VAL_SMALL_SCROLL_BARS);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_PRIM_COMMENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_MAX_ENTRY_LENGTH, 600);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_ENTER_IS_NEWLINE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_FIRST_VISIBLE_LINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_EXTRA_LINES, -1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_HEIGHT, 53);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_WIDTH, 353);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_TOP, 669);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_LEFT, 363);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, LIST_EDIT_PRIM_COMMENT, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_LIST_COMMENT
    LIST_EDIT_LIST_COMMENT = NewCtrl (hPanel, CTRL_TEXT_BOX, "", 651, 464);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_CONSTANT_NAME, "LIST_COMMENT");
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_SCROLL_BARS, VAL_BOTH_SCROLL_BARS);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_SCROLL_BAR_SIZE, VAL_SMALL_SCROLL_BARS);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_LIST_COMMENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_MAX_ENTRY_LENGTH, 600);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_ENTER_IS_NEWLINE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_FIRST_VISIBLE_LINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_EXTRA_LINES, -1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_HEIGHT, 53);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_WIDTH, 292);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_TOP, 660);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_LEFT, 400);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, LIST_EDIT_LIST_COMMENT, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_LAST_FILE
    LIST_EDIT_LAST_FILE = NewCtrl (hPanel, CTRL_STRING, "LastFile", 98, 465);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_CONSTANT_NAME, "LAST_FILE");
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_WIDTH, 39);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_LAST_FILE);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_WIDTH, 290);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_TOP, 84);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_LEFT, 700);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, LIST_EDIT_LAST_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_PLIST_REP
    LIST_EDIT_PLIST_REP = NewCtrl (hPanel, CTRL_NUMERIC, "Repetitions", 89, 378);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_CONSTANT_NAME, "PLIST_REP");
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_FRAME_COLOR, 0XCC9966);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_WIDTH, 56);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_PRECISION, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_CHECK_RANGE, VAL_IGNORE);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, LIST_EDIT_PLIST_REP);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_TOP, 75);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_LEFT, 386);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_PLIST_REP, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_2
    LIST_EDIT_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 41, 566);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_DFLT_VALUE, "ListLength");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_WIDTH, 52);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_3
    LIST_EDIT_TEXTMSG_3 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 76, 101);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_CONSTANT_NAME, "TEXTMSG_3");
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_VISIBLE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_COLOR, VAL_RED);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_DFLT_VALUE, "RING FILLED IN fillPrimParams.c");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_3);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_WIDTH, 159);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_3, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_4
    LIST_EDIT_TEXTMSG_4 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 140);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_CONSTANT_NAME, "TEXTMSG_4");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_DFLT_VALUE, " PRIMITIVE ");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_4);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_WIDTH, 103);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_4, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG
    LIST_EDIT_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 22, 554);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_DFLT_VALUE, " MASTER LIST ");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_WIDTH, 129);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_DECORATION_3
    LIST_EDIT_DECORATION_3 = NewCtrl (hPanel, CTRL_RAISED_BOX, "", 24, 363);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_CONSTANT_NAME, "DECORATION_3");
//  SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_FRAME_COLOR, 0XE6D1B3);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_HEIGHT, 122);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_WIDTH, 101);

    // Build control: LIST_EDIT_TEXTMSG_6
    LIST_EDIT_TEXTMSG_6 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 204, 448);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_CONSTANT_NAME, "TEXTMSG_6");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_6);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_WIDTH, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_6, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_7
    LIST_EDIT_TEXTMSG_7 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 289, 449);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_CONSTANT_NAME, "TEXTMSG_7");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_7);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_WIDTH, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_7, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_8
    LIST_EDIT_TEXTMSG_8 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 231, 451);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_CONSTANT_NAME, "TEXTMSG_8");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_8);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_WIDTH, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_8, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_9
    LIST_EDIT_TEXTMSG_9 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 177, 451);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_CONSTANT_NAME, "TEXTMSG_9");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_COLOR, VAL_OFFWHITE);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_9);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_WIDTH, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_9, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_5
    LIST_EDIT_TEXTMSG_5 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 159, 437);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_CONSTANT_NAME, "TEXTMSG_5");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_COLOR, 0X5F8BA0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_DFLT_VALUE, "Ctrl+");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_5);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_WIDTH, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_5, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_11
    LIST_EDIT_TEXTMSG_11 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 271, 437);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_CONSTANT_NAME, "TEXTMSG_11");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_COLOR, 0X5F8BA0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_DFLT_VALUE, "Ctrl+");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_WIDTH, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_11, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_12
    LIST_EDIT_TEXTMSG_12 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 261, 367);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_CONSTANT_NAME, "TEXTMSG_12");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_COLOR, 0X5F8BA0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_DFLT_VALUE, "ViewPrim");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_12);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_WIDTH, 46);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_12, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_13
    LIST_EDIT_TEXTMSG_13 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 257, 411);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_CONSTANT_NAME, "TEXTMSG_13");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_COLOR, 0X5F8BA0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_13);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_WIDTH, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_13, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_14
    LIST_EDIT_TEXTMSG_14 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 267, 412);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_CONSTANT_NAME, "TEXTMSG_14");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_COLOR, 0X5F8BA0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_FONT, "Wingdings");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_14);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_HEIGHT, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_WIDTH, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_14, "", VAL_DS_READ, NULL, NULL);

    // Build control: LIST_EDIT_TEXTMSG_10
    LIST_EDIT_TEXTMSG_10 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 306, 421);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_CONSTANT_NAME, "TEXTMSG_10");
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_CTRL_MODE, VAL_INDICATOR);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_COLOR, 0X5F8BA0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_DFLT_VALUE, "Ctrl+Del");
    DefaultCtrl (hPanel, LIST_EDIT_TEXTMSG_10);
//  SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, LIST_EDIT_TEXTMSG_10, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_14, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_13, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_12, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_11, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_10, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_5, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_9, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_8, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_7, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_6, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_ZPLANE_POSITION, 16);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_ZPLANE_POSITION, 17);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_4, ATTR_ZPLANE_POSITION, 18);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_3, ATTR_ZPLANE_POSITION, 19);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_ZPLANE_POSITION, 20);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_ZPLANE_POSITION, 21);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_ZPLANE_POSITION, 22);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_ZPLANE_POSITION, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_ZPLANE_POSITION, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_ZPLANE_POSITION, 25);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_ZPLANE_POSITION, 26);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_ZPLANE_POSITION, 27);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG_2, ATTR_ZPLANE_POSITION, 28);
    SetCtrlAttribute (hPanel, LIST_EDIT_TEXTMSG, ATTR_ZPLANE_POSITION, 29);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_ZPLANE_POSITION, 30);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_ZPLANE_POSITION, 31);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_ZPLANE_POSITION, 32);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_ZPLANE_POSITION, 33);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_ZPLANE_POSITION, 34);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_ZPLANE_POSITION, 35);
    SetCtrlAttribute (hPanel, LIST_EDIT_DECORATION_3, ATTR_ZPLANE_POSITION, 36);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_ZPLANE_POSITION, 37);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_ZPLANE_POSITION, 38);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_ZPLANE_POSITION, 39);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_ZPLANE_POSITION, 40);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_SLAVE, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_RING_MASTER, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LCOMM, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_PCOMM, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, LIST_EDIT_CLEAR_LIST, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, LIST_EDIT_SAVE_LIST, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, LIST_EDIT_LOAD_LIST, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, LIST_EDIT_INSERT_LIST, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, LIST_EDIT_SEND_LIST, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLACE, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_AFTER, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, LIST_EDIT_ADD_BEFORE, ATTR_CTRL_TAB_POSITION, 11);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPLIST_LENGTH, ATTR_CTRL_TAB_POSITION, 12);
    SetCtrlAttribute (hPanel, LIST_EDIT_REPPRIM_COUNT, ATTR_CTRL_TAB_POSITION, 13);
    SetCtrlAttribute (hPanel, LIST_EDIT_ALL_PRIM_COUNT, ATTR_CTRL_TAB_POSITION, 14);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COUNT, ATTR_CTRL_TAB_POSITION, 15);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIMLIST_LENGTH, ATTR_CTRL_TAB_POSITION, 16);
    SetCtrlAttribute (hPanel, LIST_EDIT_EDIT_PRIMITIVE, ATTR_CTRL_TAB_POSITION, 17);
    SetCtrlAttribute (hPanel, LIST_EDIT_DELETE_PRIMITIVE, ATTR_CTRL_TAB_POSITION, 18);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_TABLE, ATTR_CTRL_TAB_POSITION, 19);
    SetCtrlAttribute (hPanel, LIST_EDIT_DSP, ATTR_CTRL_TAB_POSITION, 20);
    SetCtrlAttribute (hPanel, LIST_EDIT_SLOT_NUMBER, ATTR_CTRL_TAB_POSITION, 21);
    SetCtrlAttribute (hPanel, LIST_EDIT_PRIM_COMMENT, ATTR_CTRL_TAB_POSITION, 22);
    SetCtrlAttribute (hPanel, LIST_EDIT_LIST_COMMENT, ATTR_CTRL_TAB_POSITION, 23);
    SetCtrlAttribute (hPanel, LIST_EDIT_LAST_FILE, ATTR_CTRL_TAB_POSITION, 24);
    SetCtrlAttribute (hPanel, LIST_EDIT_PLIST_REP, ATTR_CTRL_TAB_POSITION, 25);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(708*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(765*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(29*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(13*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_TOP, VAL_AUTO_CENTER);
    SetPanelAttribute (hPanel, ATTR_LEFT, VAL_AUTO_CENTER);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    buildMenu_primListEditor (hPanel);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildPanel_sdspListEditor ():  Build UI object SLAVE_LST; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildPanel_sdspListEditor (int hParentPanel)
{
    int hPanel;
    int error = 0;
    int hMenubar;
    int screenHeight;
    int screenWidth;
    int resAdjustment;
    int monitorId;
    int bitmapId = 0; // needed if there are bitmaps 
    char *dataPtr = NULL; 
    float SFv;
    float SFh;

    // Create the panel
    hPanel = NewPanel (hParentPanel, "SlaveList", 126, 477, 548, 390);

    // Set the panel's attributes
    SetPanelAttribute (hPanel, ATTR_CONSTANT_NAME, "SLAVE_LST");
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, 0);
//  SetPanelAttribute (hPanel, ATTR_DIMMED, 0);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_SCROLL_BARS, VAL_NO_SCROLL_BARS);
    SetPanelAttribute (hPanel, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetPanelAttribute (hPanel, ATTR_HSCROLL_OFFSET, 0);
    SetPanelAttribute (hPanel, ATTR_BACKCOLOR, 0XD5C9C4);
    SetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_THICKNESS, 16);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLEBAR_STYLE, VAL_CLASSIC);
// ****WARNING****  Not equipped to set panel's ATTR_SCROLL_BAR_STYLE = VAL_CLASSIC
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_COLOR, VAL_BLACK);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_TITLE_BACKCOLOR, VAL_LT_GRAY);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FRAME_STYLE, VAL_RAISED_OUTLINED_FRAME);
    if (hParentPanel)
        SetPanelAttribute (hPanel, ATTR_FRAME_THICKNESS, 1);
    SetPanelAttribute (hPanel, ATTR_SIZABLE, 0);
    SetPanelAttribute (hPanel, ATTR_MOVABLE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_FONT, "VAL_DIALOG_META_FONT");
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_ITALIC, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_UNDERLINE, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_STRIKEOUT, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_POINT_SIZE, 11);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_BOLD, 0);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_PARENT_SHARES_SHORTCUT_KEYS, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, 1);
    SetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_HEIGHT_FOR_SCALING, 0);
    SetPanelAttribute (hPanel, ATTR_MIN_WIDTH_FOR_SCALING, 0);
//  if (!hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_FLOATING, VAL_FLOAT_NEVER);
//  SetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, 1);
//  SetPanelAttribute (hPanel, ATTR_SYSTEM_MENU_VISIBLE, 1);
//  if (hParentPanel)
    //  SetPanelAttribute (hPanel, ATTR_TITLE_SIZE_TO_FONT, 1);
//  SetPanelAttribute (hPanel, ATTR_HAS_TASKBAR_BUTTON, 0);
//  SetPanelAttribute (hPanel, ATTR_VSCROLL_OFFSET, 0);

    // Build control: SLAVE_LST_REPLIST_LENGTH
    SLAVE_LST_REPLIST_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "RepList", 52, 187);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_CONSTANT_NAME, "REPLIST_LENGTH");
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_WIDTH, 46);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_REPLIST_LENGTH);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_TOP, 54);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_LEFT, 138);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_REPLIST_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_REPPRIM_COUNT
    SLAVE_LST_REPPRIM_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "", 52, 262);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_CONSTANT_NAME, "REPPRIM_COUNT");
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_WIDTH, 10);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_REPPRIM_COUNT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_TOP, 38);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_LEFT, 294);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_REPPRIM_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_CLEAR_LIST
    SLAVE_LST_CLEAR_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE LIST", 366, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_CONSTANT_NAME, "CLEAR_LIST");
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_CLEAR_LIST);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_SAVE_LIST
    SLAVE_LST_SAVE_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "SAVE sLIST", 330, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_CONSTANT_NAME, "SAVE_LIST");
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_SAVE_LIST);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_LOAD_LIST
    SLAVE_LST_LOAD_LIST = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "INSERT sLIST", 303, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_CONSTANT_NAME, "LOAD_LIST");
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_LOAD_LIST);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_PRIM_COUNT
    SLAVE_LST_PRIM_COUNT = NewCtrl (hPanel, CTRL_NUMERIC, "PrimCount", 33, 262);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_CONSTANT_NAME, "PRIM_COUNT");
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_WIDTH, 51);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_INCR_VALUE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_FORMAT, VAL_DECIMAL_FORMAT);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_SHOW_RADIX, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_PRIM_COUNT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_SHOW_INCDEC_ARROWS, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_WIDTH, 40);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_TOP, 20);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_LEFT, 257);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_PRIM_COUNT, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_REPLACE
    SLAVE_LST_REPLACE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "REPLACE>>", 102, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_CONSTANT_NAME, "REPLACE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_REPLACE);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_ADD_AFTER
    SLAVE_LST_ADD_AFTER = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD AFTER>>", 129, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_CONSTANT_NAME, "ADD_AFTER");
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_ADD_AFTER);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_ADD_BEFORE
    SLAVE_LST_ADD_BEFORE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "ADD BEFORE>", 75, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_CONSTANT_NAME, "ADD_BEFORE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_CTRL_MODE, VAL_HOT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_ADD_BEFORE);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_PRIMLIST_LENGTH
    SLAVE_LST_PRIMLIST_LENGTH = NewCtrl (hPanel, CTRL_NUMERIC, "PrimList", 33, 187);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_CONSTANT_NAME, "PRIMLIST_LENGTH");
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_SHORTCUT_KEY, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_WIDTH, 47);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_MAX_VALUE, -1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_MIN_VALUE, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_INCR_VALUE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_PRECISION, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_PADDING, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_SHOW_RADIX, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_PRIMLIST_LENGTH);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_WIDTH, 70);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_TOP, 36);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_LEFT, 138);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_TEXT_JUSTIFY, VAL_CENTER_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_PRIMLIST_LENGTH, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_LIST_TABLE
    SLAVE_LST_LIST_TABLE = NewCtrl (hPanel, CTRL_TABLE, "", 92, 100);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CONSTANT_NAME, "LIST_TABLE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_SCROLL_BARS, VAL_VERT_SCROLL_BAR);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_SCROLL_BAR_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_SCROLL_BAR_SIZE, VAL_LARGE_SCROLL_BARS);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CTRL_MODE, VAL_NORMAL);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_TEXT, "");
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_JUSTIFY, VAL_CENTER_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_WIDTH, 9);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_BGCOLOR, 0XB0B0B0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_MAX_VALUE, HUGE_VAL);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_MIN_VALUE, -HUGE_VAL);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_INCR_VALUE, 1.000000);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_PRECISION, 2);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CHECK_RANGE, VAL_NOTIFY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_PADDING, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_SHOW_INCDEC_ARROWS, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_WRAP_MODE, VAL_LINE_WRAP);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ENTER_IS_NEWLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_NO_EDIT_TEXT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_UPPER_LEFT_CORNER_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TABLE_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ROW_LABELS_COLOR, VAL_LT_GRAY);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_TABLE_MODE, VAL_COLUMN);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ROW_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ROW_LABELS_WIDTH, 22);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ENABLE_ROW_SIZING, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_COLUMN_LABELS_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_COLUMN_LABELS_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_COLUMN_LABELS_HEIGHT, 25);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ENABLE_COLUMN_SIZING, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ENABLE_POPUP_MENU, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CELL_TYPE, VAL_CELL_STRING);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CELL_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CELL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_STR_CELL_DFLT_VALUE, "");
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CELL_JUSTIFY, VAL_TOP_LEFT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_VERTICAL_GRID_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_HEIGHT, 445);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_WIDTH, 289);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_TOP, 71);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_LEFT, 238);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    InsertTableColumns (hPanel, SLAVE_LST_LIST_TABLE, -1, 3, VAL_USE_MASTER_CELL_TYPE);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_TEXT, "ID           PRIMITIVE");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_DATA_TYPE, VAL_DOUBLE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_MAX_VALUE, HUGE_VAL);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_MIN_VALUE, -HUGE_VAL);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_INCR_VALUE, 1.000000);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_FORMAT, VAL_FLOATING_PT_FORMAT);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_PRECISION, 2);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_COLUMN_WIDTH, 169);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_CELL_TYPE, VAL_CELL_STRING);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_NUM_CELL_DFLT_VALUE, 0.000000);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_CELL_JUSTIFY, VAL_CENTER_LEFT_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 1, ATTR_VERTICAL_GRID_VISIBLE, 1);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_TEXT, "PrimLn");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_MAX_VALUE, -1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_MIN_VALUE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_INCR_VALUE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_PRECISION, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_COLUMN_WIDTH, 39);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_CELL_TYPE, VAL_CELL_NUMERIC);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_NUM_CELL_DFLT_VALUE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_CELL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 2, ATTR_VERTICAL_GRID_VISIBLE, 1);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_TEXT, "RepLn");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_FONT, "NIDialogMetaFont");
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_ITALIC, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_UNDERLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_STRIKEOUT, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_POINT_SIZE, 11);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_BOLD, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_FONT, "NIDialogMetaFont");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_ITALIC, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_UNDERLINE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_STRIKEOUT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_POINT_SIZE, 11);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_BOLD, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_MAX_VALUE, -1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_MIN_VALUE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_INCR_VALUE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_FORMAT, VAL_HEX_FORMAT);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_PRECISION, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_CHECK_RANGE, VAL_NOTIFY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_PADDING, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_SHOW_INCDEC_ARROWS, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_MAX_ENTRY_LENGTH, -1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_ENTER_IS_NEWLINE, 0);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_NO_EDIT_TEXT, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_USE_LABEL_TEXT, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
//  SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_LABEL_WRAP_MODE, VAL_LINE_WRAP);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_COLUMN_WIDTH, 36);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_CELL_TYPE, VAL_CELL_NUMERIC);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_CELL_DIMMED, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_CELL_MODE, VAL_INDICATOR);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_HORIZONTAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_HORIZONTAL_GRID_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_MIN_NUM_LINES_VISIBLE, 1);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_INCDEC_ARROW_COLOR, VAL_LT_GRAY);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_NUM_CELL_DFLT_VALUE, 0);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_STR_CELL_DFLT_VALUE, "");
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_CELL_JUSTIFY, VAL_CENTER_CENTER_JUSTIFIED);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_VERTICAL_GRID_COLOR, VAL_BLACK);
    SetTableColumnAttribute (hPanel, SLAVE_LST_LIST_TABLE, 3, ATTR_VERTICAL_GRID_VISIBLE, 1);
    DefaultCtrl (hPanel, SLAVE_LST_LIST_TABLE);
    SetActiveTableCell (hPanel, SLAVE_LST_LIST_TABLE, MakePoint (3, 0));
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_VSCROLL_OFFSET, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindTableCellRange (hPanel, SLAVE_LST_LIST_TABLE, MakeRect(0, 0, 0, 0), "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_SLIST_DONE
    SLAVE_LST_SLIST_DONE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, " DONE ", 493, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_CONSTANT_NAME, "SLIST_DONE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_SLIST_DONE);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_EDIT_PRIMITIVE
    SLAVE_LST_EDIT_PRIMITIVE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "<<EDIT PRIM", 187, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_CONSTANT_NAME, "EDIT_PRIMITIVE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_COLOR, 0X000099);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_EDIT_PRIMITIVE);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_DELETE_PRIMITIVE
    SLAVE_LST_DELETE_PRIMITIVE = NewCtrl (hPanel, CTRL_SQUARE_COMMAND_BUTTON, "DELETE PRIM", 223, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_CONSTANT_NAME, "DELETE_PRIMITIVE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_CALLBACK_FUNCTION_POINTER, listEditWindowButtons);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_CTRL_MODE, VAL_HOT);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_COLOR, 0X990000);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_POINT_SIZE, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_BOLD, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_LABEL_VISIBLE, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_DFLT_VALUE, 0);
    DefaultCtrl (hPanel, SLAVE_LST_DELETE_PRIMITIVE);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_CMD_BUTTON_COLOR, 0XCC9966);
//  SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_AUTO_SIZING, VAL_GROW_ONLY);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_HEIGHT, 23);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_WIDTH, 98);

    // Build control: SLAVE_LST_LAST_FILE
    SLAVE_LST_LAST_FILE = NewCtrl (hPanel, CTRL_STRING, "LastFile", 73, 101);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_CONSTANT_NAME, "LAST_FILE");
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_FRAME_COLOR, VAL_LT_GRAY);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_BOLD, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_JUSTIFY, VAL_LEFT_JUSTIFIED);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_WIDTH, 39);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_HEIGHT, 15);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_COLOR, 0X000099);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_BGCOLOR, VAL_WHITE);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_DFLT_VALUE, "");
    DefaultCtrl (hPanel, SLAVE_LST_LAST_FILE);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_MAX_ENTRY_LENGTH, -1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_NO_EDIT_TEXT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_WIDTH, 288);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_TOP, 59);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_LEFT, 331);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_LABEL_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_TEXT_JUSTIFY, VAL_RIGHT_JUSTIFIED);
//  SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_HSCROLL_OFFSET, 0);
//  DSBindCtrl (hPanel, SLAVE_LST_LAST_FILE, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_TEXTMSG_2
    SLAVE_LST_TEXTMSG_2 = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 20, 194);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_CONSTANT_NAME, "TEXTMSG_2");
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_COLOR, VAL_BLACK);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_FONT, VAL_DIALOG_META_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_ITALIC, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_UNDERLINE, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_STRIKEOUT, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_POINT_SIZE, 11);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_BOLD, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_DFLT_VALUE, "ListLength");
    DefaultCtrl (hPanel, SLAVE_LST_TEXTMSG_2);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_HEIGHT, 15);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_WIDTH, 52);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_TEXTMSG_2, "", VAL_DS_READ, NULL, NULL);

    // Build control: SLAVE_LST_TEXTMSG
    SLAVE_LST_TEXTMSG = NewCtrl (hPanel, CTRL_TEXT_MSG, "", 0, 191);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_CONSTANT_NAME, "TEXTMSG");
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_VISIBLE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_CALLBACK_FUNCTION_POINTER, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_SHORTCUT_KEY, 0);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_DIMMED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_CTRL_MODE, VAL_INDICATOR);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_COLOR, VAL_BLACK);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_FONT, VAL_DIALOG_FONT);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_ITALIC, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_UNDERLINE, 1);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_STRIKEOUT, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_POINT_SIZE, 16);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_BOLD, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_BGCOLOR, VAL_TRANSPARENT);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_DFLT_VALUE, " SLAVE LIST ");
    DefaultCtrl (hPanel, SLAVE_LST_TEXTMSG);
//  SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_RAISED, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_HEIGHT, 22);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_WIDTH, 114);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_SIZE_TO_TEXT, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_TEXT_JUSTIFY, VAL_LEFT_JUSTIFIED);
//  DSBindCtrl (hPanel, SLAVE_LST_TEXTMSG, "", VAL_DS_READ, NULL, NULL);

    // Set up ZPLANE order if necessary
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_ZPLANE_POSITION, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_ZPLANE_POSITION, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_ZPLANE_POSITION, 2);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_ZPLANE_POSITION, 3);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_ZPLANE_POSITION, 4);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_ZPLANE_POSITION, 5);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_ZPLANE_POSITION, 6);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_ZPLANE_POSITION, 7);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_ZPLANE_POSITION, 8);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_ZPLANE_POSITION, 9);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_ZPLANE_POSITION, 10);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG_2, ATTR_ZPLANE_POSITION, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_TEXTMSG, ATTR_ZPLANE_POSITION, 12);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_ZPLANE_POSITION, 13);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_ZPLANE_POSITION, 14);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_ZPLANE_POSITION, 15);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_ZPLANE_POSITION, 16);

    // Set up TAB order if necessary
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLIST_LENGTH, ATTR_CTRL_TAB_POSITION, 0);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPPRIM_COUNT, ATTR_CTRL_TAB_POSITION, 1);
    SetCtrlAttribute (hPanel, SLAVE_LST_CLEAR_LIST, ATTR_CTRL_TAB_POSITION, 2);
    SetCtrlAttribute (hPanel, SLAVE_LST_SAVE_LIST, ATTR_CTRL_TAB_POSITION, 3);
    SetCtrlAttribute (hPanel, SLAVE_LST_LOAD_LIST, ATTR_CTRL_TAB_POSITION, 4);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIM_COUNT, ATTR_CTRL_TAB_POSITION, 5);
    SetCtrlAttribute (hPanel, SLAVE_LST_REPLACE, ATTR_CTRL_TAB_POSITION, 6);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_AFTER, ATTR_CTRL_TAB_POSITION, 7);
    SetCtrlAttribute (hPanel, SLAVE_LST_ADD_BEFORE, ATTR_CTRL_TAB_POSITION, 8);
    SetCtrlAttribute (hPanel, SLAVE_LST_PRIMLIST_LENGTH, ATTR_CTRL_TAB_POSITION, 9);
    SetCtrlAttribute (hPanel, SLAVE_LST_LIST_TABLE, ATTR_CTRL_TAB_POSITION, 10);
    SetCtrlAttribute (hPanel, SLAVE_LST_SLIST_DONE, ATTR_CTRL_TAB_POSITION, 11);
    SetCtrlAttribute (hPanel, SLAVE_LST_EDIT_PRIMITIVE, ATTR_CTRL_TAB_POSITION, 12);
    SetCtrlAttribute (hPanel, SLAVE_LST_DELETE_PRIMITIVE, ATTR_CTRL_TAB_POSITION, 13);
    SetCtrlAttribute (hPanel, SLAVE_LST_LAST_FILE, ATTR_CTRL_TAB_POSITION, 14);

    // Finalize panel colors, positioning, and sizing
    SetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, 0);
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 1);
    GetSystemAttribute (ATTR_RESOLUTION_ADJUSTMENT, &resAdjustment);
    if (resAdjustment == VAL_USE_PANEL_SETTING)
        resAdjustment = 0;
    if (resAdjustment)
        {
        GetMonitorFromPanel (hPanel, &monitorId);
        GetMonitorAttribute (monitorId, ATTR_HEIGHT, &screenHeight);
        GetMonitorAttribute (monitorId, ATTR_WIDTH, &screenWidth);
        SFv = (((float)screenHeight/1024)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_HEIGHT, (int)(548*SFv));
        SFh = (((float)screenWidth/1280)-1)*0.01*resAdjustment+1;
        SetPanelAttribute (hPanel, ATTR_WIDTH, (int)(390*SFh));
        SetPanelAttribute (hPanel, ATTR_TOP, (int)(126*SFv));
        SetPanelAttribute (hPanel, ATTR_LEFT, (int)(477*SFh));
        }
    SetPanelAttribute (hPanel, ATTR_SCALE_CONTENTS_ON_RESIZE, 0);

    //We're done!
    return hPanel;
Error:
    if (bitmapId)
        DiscardBitmap (bitmapId);
    return error;
}

//----------------------------------------------------------------------------------------------------
// buildMenu_primListEditor ():  Build UI object LIST_EDITM; return handle or standard UI error code.
//----------------------------------------------------------------------------------------------------
int buildMenu_primListEditor (int hParentPanel)
{
    int hMenubar;
    int error = 0;

    // Create the menubar
    hMenubar = NewMenuBar (hParentPanel);
    SetMenuBarAttribute (hMenubar, 0, ATTR_CONSTANT_NAME, "LIST_EDITM");

    // Build menu: LIST_EDITM_GOTO_MAIN
    LIST_EDITM_GOTO_MAIN = NewMenu (hMenubar, "Main", -1);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_GOTO_MAIN, ATTR_CONSTANT_NAME, "GOTO_MAIN");
//  SetMenuBarAttribute (hMenubar, LIST_EDITM_GOTO_MAIN, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_GOTO_MAIN, ATTR_CALLBACK_FUNCTION_POINTER, listEditMenu);

    // Build menu: LIST_EDITM_COMM_LIST_EDIT
    LIST_EDITM_COMM_LIST_EDIT = NewMenu (hMenubar, "CommandListEditor", -1);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_COMM_LIST_EDIT, ATTR_CONSTANT_NAME, "COMM_LIST_EDIT");
//  SetMenuBarAttribute (hMenubar, LIST_EDITM_COMM_LIST_EDIT, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_COMM_LIST_EDIT, ATTR_CALLBACK_FUNCTION_POINTER, listEditMenu);

    // Build menu: LIST_EDITM_DATA_FILE_CREATOR
    LIST_EDITM_DATA_FILE_CREATOR = NewMenu (hMenubar, "DataFileCreator", -1);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_DATA_FILE_CREATOR, ATTR_CONSTANT_NAME, "DATA_FILE_CREATOR");
//  SetMenuBarAttribute (hMenubar, LIST_EDITM_DATA_FILE_CREATOR, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_DATA_FILE_CREATOR, ATTR_CALLBACK_FUNCTION_POINTER, listEditMenu);

    // Build menu: LIST_EDITM_REVISION_UPDATE
    LIST_EDITM_REVISION_UPDATE = NewMenu (hMenubar, "RevisionUpdate", -1);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_REVISION_UPDATE, ATTR_CONSTANT_NAME, "REVISION_UPDATE");
//  SetMenuBarAttribute (hMenubar, LIST_EDITM_REVISION_UPDATE, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_REVISION_UPDATE, ATTR_CALLBACK_FUNCTION_POINTER, listEditMenu);

    // Build menu: LIST_EDITM_CLOSE
    LIST_EDITM_CLOSE = NewMenu (hMenubar, "HideWindow", -1);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_CLOSE, ATTR_CONSTANT_NAME, "CLOSE");
//  SetMenuBarAttribute (hMenubar, LIST_EDITM_CLOSE, ATTR_DIMMED, 0);
    SetMenuBarAttribute (hMenubar, LIST_EDITM_CLOSE, ATTR_CALLBACK_FUNCTION_POINTER, listEditMenu);

    //We're done!
    return hMenubar;
Error:
    return error;
}
